/*

  Copyright (C) 2001 Laurent Martelli <laurent@aopsys.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

package jac.aspects.associations;

import jac.core.*;
import jac.core.rtti.*;
import jac.util.Log;
/*
import java.lang.reflect.*;
import java.util.*;
*/

public class RelationWrapper extends Wrapper {

   public Object updateOppositeRelations() 
   {
      if (attr("rtti.updateOpposite")!=null)
         return proceed();
      Log.trace("associations","updateOppositeRelations "+
                method()+"("+(arg(0)!=null?arg(0).getClass().getName():"null")+")");
      ClassItem cli = ClassRepository.get().getClass(wrappee().getClass());
      MethodItem method = cli.getMethod(method());

      // References
      FieldItem[] fields = method.getWrittenFields();
      FieldItem opposite;
      Object newValue = arg(0);
      for( int i = 0; fields != null && i < fields.length; i++ ) {
         FieldItem field = fields[i];
         Object currentValue = field.get(wrappee());
         opposite = (FieldItem)field.getAttribute("rtti.oppositeRelation");
         attrdef("rtti.updateOpposite",opposite);
         Log.trace("associations","UpdateOppositeRelation in "+method()+" : "+
                   fields[i].getName()+" -> "+opposite.getName());
         if (opposite instanceof CollectionItem) {
            if (currentValue!=null)
               ((CollectionItem)opposite).remove(currentValue,wrappee(),null);
            if (newValue!=null)
               ((CollectionItem)opposite).add(newValue,wrappee(),null);
         } else {
            if (currentValue!=null) {
               opposite.setThroughWriter(currentValue,null);
            }
            opposite.setThroughWriter(newValue,wrappee());
         }
      }

      // Adders
      CollectionItem[] collections = method.getAddedCollections();
      Object addedValue = arg(0);
      for( int i = 0; collections != null && i < collections.length; i++ ) {
         CollectionItem collection = collections[i];
         opposite = (FieldItem)collection.getAttribute("rtti.oppositeRelation");
         if (opposite==null) {
            Log.error(collection.toString()+" has no opposite relation");
            continue;
         }
         Log.trace("associations","UpdateOppositeRelation in "+method()+" : "+
                   collection.getName()+" -> "+opposite.getName());
         attrdef("rtti.updateOpposite",opposite);
         if (opposite instanceof CollectionItem) {
            ((CollectionItem)opposite).add(addedValue,wrappee(),null);
         } else {
            if (addedValue!=null) {
               Object currentValue = opposite.get(addedValue);
               if (currentValue!=null) {
                  collection.remove(currentValue,addedValue,null);
               }
               opposite.setThroughWriter(addedValue,wrappee());
            }
         }
      }
      
      // Removers
      collections = method.getRemovedCollections();
      Object removedValue = arg(0);
      for( int i = 0; collections != null && i < collections.length; i++ ) {
         CollectionItem collection = collections[i];
         Log.trace("associations","removed collection "+collection);
         opposite = (FieldItem)collection.getAttribute("rtti.oppositeRelation");
         attrdef("rtti.updateOpposite",opposite);
         if (opposite==null) {
            Log.error(collection.toString()+" has no opposite relation");
            continue;
         }
         Log.trace("associations","UpdateOppositeRelation in "+method()+" : "+
                   collection.getName()+" -> "+opposite.getName());
         if (removedValue != null) {
            if (opposite instanceof CollectionItem) {
               ((CollectionItem)opposite).remove(removedValue,wrappee(),null);
            } else {
               opposite.setThroughWriter(removedValue,null);
            }
         }
      }

      Object result = proceed();
      return result;
   }   
}


