/*
  Copyright (C) 2001 Renaud Pawlak, Laurent Martelli
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

package jac.aspects.authentication;

import java.util.*;
import jac.core.*;
import jac.util.*;
import jac.core.rtti.*;

/**
 * This AC weaves the authentication aspect.
 *
 * <p>It allows the programmer to define some trusted users and to
 * restrict the access of the methods that need to be authenticated.
 *
 * <p>It can be used with some owning features.<p>
 *
 * @see OwningWrapper
 * @see User */

public class AuthenticationAC extends AspectComponent implements AuthenticationConf {   

   /** Stores the trusted users. */
   protected Hashtable trustedUsers = new Hashtable();
   
   /**
    * Tells if a given user is trusted or not.
    *
    * @param username the user's name
    * @return true if the user has been added to the trusted users
    * list */
   
   public boolean isTrustedUser(String username, String password) 
   {
      Log.trace("authentication","isTrustedUser("+username+")");
      Object userPassword = trustedUsers.get(username);
      if (userPassword != null)
         return ( userPassword.equals(password) );
      else
         return false;
   }

   public void addTrustedUser(String username, String password) {
      Log.trace("authentication","addTrustedUser("+username+","+password+")");
      trustedUsers.put(username, password);
   }

   /**
    * Returns all the declared trusted users.
    *
    * @see #addTrustedUser(String,String) */

   public Hashtable getTrustedUsers() {
      return trustedUsers;
   }
   
   public void addRestrictedMethods(String classes,
                                    String methods,
                                    String objects ) {
      Log.trace("authentication","addRestrictedMethods("+
                classes+","+methods+","+objects+")");
      pointcut(objects,classes,methods,AuthenticationWrapper.class.getName(),
               "authenticate",false);
   }

    public void addRestrictedObjects(String objects) {
       pointcut(objects,"ALL","ALL",AuthenticationWrapper.class.getName(),
                "authenticate",false);
   }

   public boolean beforeRunningWrapper ( Wrapper wrapper, 
                                         String wrappingMethod ) {
      // skip the authentication aspect if already performed
      if( attr("Authentication.done") != null ) {
         Log.trace("authentication", "skiping the authentication aspect");
         return false;
      }
      return true;
   }

   public void afterRunningWrapper ( Wrapper wrapper, 
                                     String wrappingMethod ) {
      // memorize the fact that authentication is OK
      attrdef("Authentication.done","");
   }


}

// Local Variables: ***
// c-basic-offset:3 ***
// End: ***
