/*
  Copyright (C) 2001 Renaud Pawlak, Laurent Martelli
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

package jac.aspects.authentication;

/**
 * This is the configuration interface of the authentication aspect.<p>
 *
 * @see AuthenticationAC
 */

public interface AuthenticationConf {   

   /**
    * This configuration method adds a trusted user in the users
    * database. 
    * @param username the user's name
    * @param password the password of the user
    */

   void addTrustedUser(String username, String password);

   /**
    * This configuration method restricts a given set of methods.
    *
    * <p>When a method is restricted, the collaboration that contains
    * the restricted method invocation must contain an attribute that
    * represents the current user. If not, the authentication aspect
    * ask the user to input its caracteristics by using the display of
    * the collaboration (or the default on if none). If the user is
    * not recognized as part of the trusted users list, then an
    * authentication exception is raised.<p>
    *
    * @param classes a class expression
    * @param objects an object expression
    * @param methods an expression matching the methods to restrict
    * @see #addRestrictedObjects(String)
    * @see AuthenticationWrapper
    */
   
   void addRestrictedMethods(String classes, String methods, String objects );

   /**
    * This configuration method restricts some objects for
    * authentication.
    *
    * @param classes a class expression
    * @param objects an object expression
    * @see #addRestrictedMethods(String,String,String)
    * @see AuthenticationWrapper */
   
   void addRestrictedObjects(String objects);

}
