/*
  Copyright (C) 2001 Renaud Pawlak, Laurent Martelli
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

package jac.aspects.authentication;

import jac.core.*;
import jac.core.rtti.*;
import jac.aspects.gui.*;
import jac.util.*;

/**
 * This wrapper provides an authentication scheme that asks the
 * user to enter its name when the collaboration does not contain the
 * current user description.
 *
 * <p>To be effective, this wrapper can be used with an owning
 * wrapper.<p>
 *
 * @see OwningWrapper */

public class AuthenticationWrapper extends Wrapper  {

   /**
    * This wrapping method authenticates a call on the wrapped
    * method.<p>
    *
    * @return the value returned by the wrapped method */

   public Object authenticate ( )
      throws AuthenticationFailedException, AccessDeniedException
   {
      Log.trace("authentication","authenticate for method "+method()+" on "+wrappee());
      Log.trace("authentication","Name is : " + (String)attr("Authentication.user") );
      String name=(String)attr("Authentication.user");
      String password = null;
      if ( name == null ) {
         try {
            MethodItem toCall = ClassRepository.get()
               .getClass(wrappee().getClass()).getMethod(method());
            MethodItem method = ClassRepository.get()
               .getClass(TrustingAuthentication.class).getMethod("authenticateWithPassword");

            TrustingAuthentication trust = new TrustingAuthentication();
            Display display = (Display)attr("Gui.display");
            if (display!=null) {

               Log.trace("authentication","showing invocation for "+method+" on "+ trust);
               Object[] parameters = new Object[method.getParameterTypes().length];
               if ( ! display.showInput( trust, method, parameters ) ) {
                  throw new AuthenticationFailedException("Action canceled by the user");
               } else {
                  name = (String) parameters[0];
                  password = (String)parameters[1];
               }
               //name = (String) trust.getUserName();
               Log.trace("authentication","done => "+name);

            } else {
               Log.trace("authentication","No display available");
               throw new AuthenticationFailedException("No display available");
            }
         } catch (Exception e) {
            e.printStackTrace();
         }
         if (name!=null) {
            attrdef("Authentication.user", name);
         } else {
            throw new AuthenticationFailedException("Unknown reason");
         }
         if (isTrustedUser(name, password)) {
            Log.trace("authentication","Accesses granted to "+name+":"+
                      password+" for "+method());
            return proceed();
         } else {
            Log.trace("authentication","Accesses denied to "+name+":"
                      +password+" for "+method());
            throw new AccessDeniedException();
         }
      } else {
         Log.trace("authentication","Accesses granted to "+name+":"+
                   password+" for "+method());
         return proceed();
      }
   }

   /**
    * Returns true if the user is in the trusted users list.
    *
    * @param username the name of the user to check
    * @return true if trusted */

   public boolean isTrustedUser(String username, String password) {
      Log.trace("authentication","Is trusted user: " + username + ":" +
                getAspectComponent() );
      return ((AuthenticationAC)getAspectComponent()).isTrustedUser(username, password);
   }

   /**
    * This exception handler reacts when the access is denied. */

   public void catchAccessDenied(AccessDeniedException e) {
      Log.trace("authentication",e.toString());
      Display display = (Display)attr("Gui.display");
      if (display!=null) {
         display.showMessage( e.toString(), "", false, false, false );
      } else {
         Log.trace("authentication","No display available");
      }      
   }
}
