/*

  Copyright (C) 2001 Renaud Pawlak, Laurent Martelli
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

package jac.aspects.authentication;

import java.util.*;
import jac.core.*;

/**
 * This wrapper defines the client-side objects extensions needed when
 * performing authentication.
 */

public class OwningWrapper extends Wrapper {
   User owner;
   
   /**
    * This role method gets the owner of the wrapped objet.
    * 
    * @return the owner
    */

   public User getOwner() 
   {
      return owner;  
   }
   
   /**
    * This role method sets the owner of the wrapped objet.
    * 
    * @param owner the new owner
    */

   public void setOwner(User owner) 
   {
      this.owner = owner;
   }

   /**
    * This wrapping method binds the owner of the wrapped object
    * depending on the context.<p>
    * 
    * @return the value returned by the wrapped method */
   
   public Object bindOwner ( )
   {
      User user = (User)attr("user");
      System.out.println ( ">>> bindOwner -> "+user.getName());
      ((Wrappee)arg(0)).invokeRoleMethod("setOwner", new Object [] {user});
      return proceed();
   }

   /**
    * This wrapping method should wrap any method that returns a set
    * of owned objects.<p>
    *
    * If the context tells that the currenly accessing client is a
    * given user, this method creates a new set of objects containing
    * only the ones that are owned by this user and returns it instead
    * of the initially returned one.<p>
    *
    * If context does not define any user. Then the returned set of
    * objects is equals to the set of object that are not owned by
    * anybody.<p>
    * 
    * @return the filtered set of owned objects */

   public Object filterCollection ( )
   {
      System.out.println(">>> filterCollection ");
      Vector objects =  (Vector)proceed();
      Vector result = new Vector();
      for(int i=0; i<objects.size(); i++)
      {
         User owner = (User) ((Wrappee)objects.get(i)).invokeRoleMethod(
            "getOwner", new Object [] {});
         if (owner==null) {
            System.out.println("*** no owner for object");
         } else if (owner.equals(attr("user"))) {
            result.add(objects.get(i));
         }
      }
      System.out.println("    "+objects.size()+" -> "+result.size());
       
      return result;
   }
}
