/*
  Copyright (C) 2001 Renaud Pawlak

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

package jac.aspects.confirmation;

import jac.core.AspectComponent;
import jac.core.Wrapper;
import jac.core.Display;
import jac.core.rtti.NamingConventions;

/**
 * The confirmation aspect implementation (allows the user to add
 * confirmation popups before committing. */

public class ConfirmationAC extends AspectComponent implements ConfirmationConf {

   static String cancellationMessage = "invocation was cancelled by the user";  


   public void confirm(String classes, String methods, String objects ) {
      pointcut(objects,classes,methods,
               new ConfirmationWrapper(),"confirm"); 
   }

   /**
    * A confirmation wrapper that wraps methods to show a confirmation
    * message box before actually performing the call. */
   
   public class ConfirmationWrapper extends Wrapper {

      /** The wrapping method. */
      public Object confirm() {
         Display display = (Display)attr("Gui.display");
         if( display != null ) {
            if(!display.showMessage("Do you really want to "+
                                    NamingConventions.textForName(method())+
                                    " for '"+wrappee()+"' ?",
                                    "Confirmation",
                                    true, true, false)) {
               throw new RuntimeException(cancellationMessage);
            }
         }
         return proceed();
      }
      
      /** The exception handler. */
      public void catchCancellation(RuntimeException re) {
         if( re.getMessage().equals(cancellationMessage) ) {
            System.out.println("input cancelled");
         }
         else {
            throw re;
         }
      } 

   }

}
