/*
  Copyright (C) 2001 Renaud Pawlak

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser Generaly Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */

package jac.aspects.distribution;

import jac.core.*;
import jac.core.dist.*;
import jac.util.Log;

import gnu.regexp.*;
import java.util.*;

/**
 * This aspect component implements a simple broadcasting aspect.
 *
 * <p>Principles: a broadcaster, located on a given host forwards some
 * calls to a set of replicas located on remote hosts.
 *
 * @see BroadcastingConf
 * @author Renaud Pawlak
 * @version 0.8.1 */

public class BroadcastingAC extends AspectComponent implements BroadcastingConf {

   public void addBroadcaster( String wrappeeName,
                               String methods,
                               String broadcasterHost, 
                               String replicasHost ) {

      pointcut( wrappeeName, ".*", methods,
                new BroadcastingWrapper( replicasHost ),
                "broadcast", broadcasterHost );
   }

   /**
    * This wrapper wraps the broadcaster with a wrapping method that
    * broadcast all the calls to the remote replicas. */

   public class BroadcastingWrapper extends Wrapper {

      Vector replicas = null;
      String hostExpr;
      boolean doFill = true;

      public BroadcastingWrapper( String hostExpr ) {
         this.hostExpr = hostExpr;
      }

      public void invalidate() {
         doFill = true;
      }

      /**
       * Performs a broadcasting. */

      public Object broadcast() {
         if( doFill ) {
            replicas = Topology.getPartialTopology(hostExpr)
               .getReplicas( this.wrappee() );
            doFill = false;
         }
         if( replicas.size() == 0 ) {
            // none replicas where found, we perform a local call and 
            // will try to get them again on the next call
            doFill = true;
            Log.warning("broadcasting: no replica found, local call performed");
            return proceed();
         }
         for( int i=0; i<replicas.size(); i++ ) {
            ((RemoteRef)replicas.get(i)).invoke(this.method(),this.args());
         }
         return proceed();
      }

   }
}
