/*
  Copyright (C) 2001 Renaud Pawlak

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */

package jac.aspects.distribution.consistency;

import jac.util.*;
import jac.core.*;
import jac.core.dist.*;
import java.util.*;
import jac.aspects.naming.*;

/**
 * This wrapper implements a client-server consistency protocol.
 *
 * <p>It is a special consistency protocol since the wrappee acts as a
 * pure client (a stub) for its known replicas.<p>
 * 
 * @version 0.8.1
 * @author <a href="http://cedric.cnam.fr/~pawlak/index-english.html">Renaud Pawlak</a> */

public class ClientServerConsistencyWrapper extends ConsistencyWrapper {

   String hosts = null;

   /**
    * A friendly constructor for a client-server consistency wrapper.
    *
    * @param hosts a regular expression that defines the host where
    * the server is located */

   public ClientServerConsistencyWrapper( String serverHost ) {
      knownReplicas = null;
      this.hosts = hosts;
   } 

   /** An empty constructor for the Consistency class. */
   public ClientServerConsistencyWrapper() {}

   /**
    * Forwards the call to the server(s).<p>
    *
    * Do not call the replica except if we do not know any server (in
    * this case, we are a server).<p>
    *
    * @return the value returned by the server */

   public Object whenCall () {

      if( knownReplicas == null ) {
         knownReplicas = Topology.getPartialTopology(hosts).getReplicas(wrappee());
      } 

      if(knownReplicas != null && knownReplicas.size() > 0 &&
         (! ((RemoteRef)knownReplicas.get(0)).getRemCont().isLocal()) ) {
         Log.trace("consistency", "(client-server) call event on " + 
                   NameRepository.get().getName(wrappee()) + 
                   ":" + method() + ":" + 
                   ((RemoteRef)knownReplicas.get(0)).getRemCont().getName());
         Object ret = ((RemoteRef)knownReplicas.get(0)).invoke(
            "invokeRoleMethod",
            new Object[] {
               "acceptRemoteCall",
               new Object[] { null, new Object[] { method(), args() } }
            }
         );
         return ret;
      } else {
         /* we do not know any replica, so we are the server... */
         return proceed();
      }
   }

   /**
    * Calls the method on the server.
    *
    * @param remoteReplica the client remote reference
    * @param data the name and the parameters of the server method
    */

   public Object acceptRemoteCall ( RemoteRef remoteReplica,
                                    Object[] data) {
      Log.trace("consistency", "(client-server) remote call event on " + 
                NameRepository.get().getName(wrappee()) +
                ":" + (String)data[0] + ":" + 
                Arrays.asList((Object[])data[1]));
      Object ret = wrappee().invoke ( (String)data[0], (Object[])data[1] );
      return ret;
   }
}









