/*

  Copyright (C) 2001 Renaud Pawlak, Laurent Martelli
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA */

package jac.aspects.gui;

import java.text.*;
import java.util.*;
import javax.swing.*;
import java.awt.Dimension;
import java.awt.Component;

import jac.util.Log;

/**
 * A Swing editor component for field values when possible values must
 * be selected within a collection of objects.  */

public class ChoiceFieldEditor extends JComboBox implements ValueEditor { 
   
   class MyComparator implements Comparator {
      public int compare(Object o1, Object o2) {
         return o1.toString().compareToIgnoreCase(o2.toString());
      }
      public boolean equals(Object o1, Object o2) {
         return o1.toString().equals(o2.toString());
      }
   }

   /**
    * Constructs a new choice field editor. */

   public ChoiceFieldEditor( Object[] choice, boolean editable) {
      super();
      try {
         Arrays.sort(choice, new MyComparator() );
      } catch (ClassCastException e) {
         Log.error("ChoiceFieldEditor : failed to sort array");
         e.printStackTrace();
      }
      for(int i=0; i<choice.length; i++) {
         addItem(choice[i]);
      }
      setEditable(editable);
  }

   /**
    * Gets the value of the edited field.
    *
    * @return an object within the choices collection */
 
   public Object getValue() {
      return ( getSelectedItem() );
   }

   /**
    * Sets the value of the edited field.
    *
    * @param value an object within the choices collection */

   public void setValue(Object value) {
      setSelectedItem(value);
   }

   public void setWidth(int width) {
      Dimension dim = getSize();
      Dimension newDim = new Dimension(width,(int)dim.getHeight());
      setSize(newDim);
   }

   public void setHeight(int height) {
      Dimension dim = getSize();
      Dimension newDim = new Dimension((int)dim.getWidth(),height);
      setSize(newDim);
   }

   public Component getValueComponent() {
      // This is a very dirty hack to get the real data component since
      // JComboBox does not fire the focus events (certainly a bug in 
      // JDK 1.3
      if(isEditable()) {
         return getComponent(2);
      } else {
         return getComponent(0);
      }
   }
   public void onSetFocus(Object extraOption) {}

}
