/*

  Copyright (C) 2001 Renaud Pawlak, Laurent Martelli
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA */

package jac.aspects.gui;

import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.lang.reflect.*;
import jac.core.*;
import jac.core.rtti.*;
import jac.aspects.naming.*;
import java.net.*;
import java.io.*;

/**
 * This class is an administration GUI for the JAC applications
 * registered within the system.
 *
 * <p>It is a good sample to know about creating customized GUIs.
 *
 * @see jac.core.ApplicationRepository
 * @see jac.core.Application */

public class ConsoleAdmin extends AbstractSwingDisplay
   implements ActionListener, MouseListener { 

   static final String newline = "/n";
   static ConsoleAdmin instance;
   String applicationName;
   JFileChooser fc = new JFileChooser();
   File propFile;
   Application application;
   JSplitPane splitPane;
   JPanel upperPane = new JPanel();
   JPanel lowerPane = new JPanel();
   Component gm;
   /*
   public static Object[] getDeclaredACNames() {
      return ((ACManager)ACManager.get()).getDeclaredACs().toArray();
      }*/

   /**
    * Constructs a view on a given JAC program.
    *
    * <p>To be compliant with this view, the JAC program must be in a
    * package that corresponds to its name. The package must contain a
    * <code>Main</code> or a <code>Run</code> class to launch the
    * program. Each aspects of the program must be in the package an
    * the naming convention imposes a name of the following form
    * "<programName><aspectName>AC".
    *
    * @param programName the program that is displayed by the viewer
    */

   public ConsoleAdmin() {

      super();

      setTitle("Application repository"); 

      UIManager.put( "Label.font", new Font( "SansSerif", Font.PLAIN, 12 ) );
      UIManager.put( "ComboBox.font", new Font( "SansSerif", Font.PLAIN, 12 ) );
      UIManager.put( "Menu.font", new Font( "SansSerif", Font.PLAIN, 12 ) );
      UIManager.put( "MenuBar.font", new Font( "SansSerif", Font.PLAIN, 12 ) );      
      UIManager.put( "Panel.font", new Font( "SansSerif", Font.PLAIN, 12 ) );
      UIManager.put( "Border.font", new Font( "SansSerif", Font.PLAIN, 12 ) );
      UIManager.put( "MenuItem.font", new Font( "SansSerif", Font.PLAIN, 12 ) );
      UIManager.put( "Button.font", new Font( "SansSerif", Font.PLAIN, 12 ) );
      UIManager.put( "RadioButton.font", new Font( "SansSerif", Font.PLAIN, 12 ) );
      UIManager.put( "RadioButtonMenuItem.font", new Font( "SansSerif", Font.PLAIN, 12 ) );
      UIManager.put( "CheckBox.font", new Font( "SansSerif", Font.PLAIN, 12 ) );      
      UIManager.put( "CheckBoxMenuItem.font", new Font( "SansSerif", Font.PLAIN, 12 ) );     
      UIManager.put( "TabbedPane.font", new Font( "SansSerif", Font.PLAIN, 12 ) );

      String[] aspectNames = new String[] {
         "naming", "binding", "tracing", "counting", "debugging",
         "gui", "persistence", "authentication", "session", "distribution" };


      instance = this;

      ClassItem applicationManagerClass = ClassRepository.get().getClass( jac.core.ApplicationRepository.class );
      ClassItem applicationClass = ClassRepository.get().getClass( jac.core.Application.class );

      applicationManagerClass.getField( "applications" ).setAttribute( "Gui.Swing.targetContainer", lowerPane );

      addWindowListener ( new WindowAdapter () {
            public void windowClosing( WindowEvent e ) {
               e.getWindow().dispose();
            }
         }
      );
      
      splitPane  = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
                                 upperPane, lowerPane);
      splitPane.setOneTouchExpandable(true);
      getContentPane().add(splitPane);

      gm = new ObjectView(this, null, ApplicationRepository.get());

     
      lowerPane.setLayout( new BoxLayout( lowerPane, BoxLayout.Y_AXIS ) );      
      upperPane.setLayout( new BoxLayout( upperPane, BoxLayout.Y_AXIS ) );      

      upperPane.add(gm);
     
      fc.addChoosableFileFilter(new ApplicationFileFilter("applications"));
      fc.setFileView(new ApplicationFileView());
    
      pack();
            

      Dimension screenDim = Toolkit.getDefaultToolkit().getScreenSize();
      setBounds( new Rectangle( 0,0,
                                (int) ((float)screenDim.getWidth())/2,
                                (int) (((float)screenDim.getHeight())*2)/3 ) );

      validate();

      show();

   }

   public void close() {}
      
   boolean isApplication( String name ) {
      return name.equals("application.prop");
   }

   final class ApplicationFileView extends javax.swing.filechooser.FileView {
   
      public Icon getIcon( java.io.File f ) {
         if ( isApplication( f.getName() ) ) {
            return new ImageIcon( ResourceManager.RESOURCES_PATH + "application.gif" );
         }
         return super.getIcon( f );
      }
   }
   
   final class ApplicationFileFilter extends FileFilter {

      String type;

      ApplicationFileFilter(String type) {
         this.type = type;
      }


      public String getDescription() { return type; }
   
      public boolean accept(File file) {
         if (file.isDirectory()) {
            return true;
         }

         return isApplication(file.getName());    
      }
      
   }

   void createMenus() {
     
      JMenuBar menuBar;
      JMenu menu, submenu;
      JMenuItem menuItem;
      JCheckBoxMenuItem cbMenuItem;
      JRadioButtonMenuItem rbMenuItem;


      //Create the menu bar.
      menuBar = new JMenuBar();
      setJMenuBar(menuBar);

      //Build SYSTEM menu.
      menu = new JMenu( "Applications"  );
      menu.setMnemonic( KeyEvent.VK_S );
      menu.getAccessibleContext().setAccessibleDescription( "la menu de lancement d'application" );
      menuBar.add(menu);

      //a application of JMenuItems
      menuItem = new JMenuItem("Open");
      menuItem.setAccelerator(KeyStroke.getKeyStroke(
         KeyEvent.VK_1, ActionEvent.ALT_MASK));
      menuItem.addActionListener(this);

      menu.add(menuItem);

      menu.addSeparator();

      menuItem = new JMenuItem( "Close" );
      menuItem.setMnemonic(KeyEvent.VK_X);
      menuItem.setAccelerator(KeyStroke.getKeyStroke(
         KeyEvent.VK_X, ActionEvent.CTRL_MASK));
      menuItem.addActionListener(this);
      menu.add(menuItem);


   } 

   /** Do nothing. */
   public void mouseClicked(MouseEvent me){}
   /** Do nothing. */
   public void mousePressed(MouseEvent me){}
   /** Do nothing. */
   public void mouseReleased(MouseEvent me){}
   /** Do nothing. */
   public void mouseExited(MouseEvent me){}
   /** Do nothing. */
   public void mouseEntered(MouseEvent me){}   
   /** Do nothing. */
   public void actionPerformed(ActionEvent ae){
      /*
      String action = ae.getActionCommand();
      
      if( action.equals("Open") )
         openApplication();
      

      else if ( action.equals("Close") ) 
         closeApplication();
      */
   }


   void openApplication() {
                
      int returnVal = fc.showOpenDialog(ConsoleAdmin.this);
      
      if (returnVal == JFileChooser.APPROVE_OPTION) {

         try {
            propFile = fc.getSelectedFile();
            if(isApplication(propFile.getName())) {
               //new Application( propFile );
            }
         } catch(Exception ioe) {
            ioe.printStackTrace();         
         }
      }
      Component cmp = new ObjectView(this, null, application);
      upperPane.add(cmp);
      getContentPane().validate();
      int size = cmp.getWidth();
      System.out.println("--"+size);
      splitPane.setDividerLocation(size);
      repaint();
   }

   private void createApplication() throws Exception {
   }
         
  
   void closeApplication(){}
   
   // Implements the Display interface

   public String getDisplayID() {
      return "application-repository";
   }

   public void setDisplayID(String displayID) {}

   public void show( Object object ) {
      if (object instanceof Throwable) {
         ((Throwable)object).printStackTrace();
      } else {
         ObjectView.addViewFor( object, this, this );
      }
   }

}



















