/*

  Copyright (C) 2001 Renaud Pawlak, Laurent Martelli
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

package jac.aspects.gui;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.List;
import java.util.Vector;
import java.lang.reflect.InvocationTargetException;
import jac.util.Log;
import jac.core.rtti.*;

/**
 * This dialog is used to ask the parameters values when a constructor is
 * called on a viewed JAC object.<p>
 *
 * @see View
 *
 * @author <a href="mailto:laurent@aopsys.com">Laurent Martelli</a>
 * @author <a href="mailto:renaud@cnam.fr">Renaud Pawlak</a>
 */

public class ConstructorBox extends JDialog implements ActionListener {

   private JButton okButton;
   private JButton cancelButton;

   Object result = null;
   
   private ConstructorItem constructor = null;
   
   /**
    * Construct a new calling box to get the parameters of the invoked
    * constructor.<p>
    *
    * @param parent the parent window (a view on a wrappee)
    * @param substance the object viewed by the parent window (the
    * object the method is called on)
    * @param constructor the invoked constructor */
   
   public ConstructorBox( Window parent, ConstructorItem constructor ) {
      super( (JFrame)parent, "New instance " + constructor.getName(), true );
      Log.trace("gui","ConstructorBox("+constructor+")");
      setModal( true );
      this.constructor = constructor;

      Container contentPane = getContentPane();

      addWindowListener ( new WindowAdapter () {
            public void windowClosing( WindowEvent e ) {
               e.getWindow().dispose();
            }
         }
      );
      
      Vector containers = new Vector();

      JPanel panel = RenderFactory.createParameters(parent,constructor,null);
      contentPane.add("Center", panel);
      JPanel p2 = new JPanel();
      p2.setBorder( BorderFactory.createEtchedBorder() );
      okButton = addButton( p2, "Ok" );
      cancelButton = addButton( p2, "Cancel" );
      contentPane.add( "South", p2 );
      pack();
      ((JPanel)panel.getComponent(0)).getComponent(1).requestFocus();
      setVisible ( true );
   }

   /**
    * For internal use.<p>
    */
   JButton addButton(Container c, String name)  {
      JButton button = new JButton(name);
      button.addActionListener(this);
      c.add(button);
      return button;
   }

   /**
    * Implements what is done when a button is pressed (may be either
    * OK or CANCEL).<p>
    *
    * @param evt tell what button was pressed */

   public void actionPerformed(ActionEvent evt) {

      try {

         Object source = evt.getSource();
         if(source == okButton) {
            Object[] v = RenderFactory.buildParameterValues(constructor,getContentPane());
            try {
               result = constructor.newInstance( v );
            } catch ( Exception e ) {
               setVisible(false);
               if ( e instanceof InvocationTargetException ) { 
                  ((InvocationTargetException)e).printStackTrace();
               } else {
                  e.printStackTrace();
               }  
            }
            dispose();
         } else if (source == cancelButton) {
            result = null;
            dispose();
         }
         
      } catch ( Exception e ) {
         e.printStackTrace ();
      }
   }

   /**
    * Returns the newly created instance.
    *
    * <p>If the construction is canceled by the user, then this method
    * is meaningless. The client should catch and handle the
    * <code>InputCanceledException</code> to treat cancelation.
    *
    * @return the newly created instance
    * @see InputCanceledException */

   public Object getResult() {
      return result;
   }
   

}
