/*

  Copyright (C) 2001 Renaud Pawlak, Laurent Martelli
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA */

package jac.aspects.gui;

import java.text.*;
import java.util.*;
import javax.swing.*;
import java.awt.Component;

/**
 * A Swing editor component for date values.
 */

public class DateEditor extends JTextField implements ValueEditor 
{
   
   /** Stores the default format of the date */
   protected SimpleDateFormat format;

   /**
    * Constructs a new date editor. */

   public DateEditor() {
      super(10);
      format = new SimpleDateFormat(GuiAC.getDateFormat());
      setText(format.format(new Date()));
   }

   /**
    * Gets the value of the edited date.
    *
    * @return a date object, null if the entered string was not well
    * formatted */
 
   public Object getValue() {
      try {
         return format.parse(getText());
      } catch (Exception e) {
         return null;
      }
   }

   /**
    * Sets the value of the edited date.
    *
    * @param value a <code>Date</code> instance */

   public void setValue(Object value) {
      if( value != null )
         setText(format.format((Date)value));
   }

   public void setWidth(int width) {
      setColumns(width);
   }

   public void setHeight(int height) {}

   public Component getValueComponent() {
      return this;
   }

   public void onSetFocus(Object extraOption) {}
}
