/*

  Copyright (C) 2001 Laurent Martelli
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA */

package jac.aspects.gui;

import java.text.*;
import java.util.*;
import java.awt.Component;
import java.awt.Color;
import javax.swing.*;
import javax.swing.table.TableCellRenderer;

import jac.util.Log;

/**
 * A Swing viewer component for date values.
 */

public class DateViewer extends JLabel implements ValueViewer, TableCellRenderer
{
   
   /** Stores the default format of the date */
   protected SimpleDateFormat format;

   /**
    * Constructs a new date editor. */

   public DateViewer() {
      super();
      format = new SimpleDateFormat(GuiAC.getDateFormat());
   }

   /**
    * Sets the value of the edited date.
    *
    * @param value a <code>Date</code> instance */

   public void setValue(Object value) {
      if (value!=null)
         setText(format.format((Date)value));
      else
         setText("");
   }

   // TableCellRenderer
   public Component getTableCellRendererComponent(JTable table, Object value, 
                                                  boolean isSelected, boolean hasFocus, 
                                                  int row, int column) 
   {
      Log.trace("gui.date","getTableCellRendererComponent("+row+","+column+","+isSelected+")");
      setOpaque(true); // so that the background is really drawn
      if (isSelected) {
         setForeground(table.getSelectionForeground());
         setBackground(table.getSelectionBackground());
      } else {
         setForeground(table.getForeground());
         setBackground(table.getBackground());
      }
      setFont(null);
      setValue(value); 

      return this;
   }
}
