/*

  Copyright (C) 2001 Laurent Martelli
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA */

package jac.aspects.gui;

/**
 * HTML editor for fields configured with setFieldChoice.
 *
 * @see GuiConf#setFieldChoice(String,String,String,String[])
 * @see GuiConf#setDynamicFieldChoice(String,String,String,String,String)
 * @see GuiConf#setMethodDynamicParameterChoice(String,String,String[],String[],String[])
 */
public class HTMLChoiceFieldEditor  extends AbstractHTMLEditor implements HTMLEditor 
{
   Object[] choices;
   boolean isEditable;

   public HTMLChoiceFieldEditor(Object[] choices, boolean isEditable) {
      this.choices = choices;
      this.isEditable = isEditable;
   }

   /*
    * HTMLEditor interface
    */

   public String genHTML(String name) {
      String html = "<SELECT NAME="+name+">\n";
      String selected;
      for (int j=0; j<choices.length; j++) {
         selected = choices[j].toString().equals(value.toString()) ? " selected" : "";
         html += "<OPTION VALUE=\""+choices[j]+"\""+selected+">"+choices[j]+"</OPTION>\n";
      }
      html += "</SELECT>\n";
      return html;
   }

   public Object getValue(String string) {
      if ( type == int.class ) {
         return  new Integer (string);
      } else if ( type == boolean.class ) {
         return  new Boolean (string);
      } else if ( type == long.class ) {
         return  new Long (string);
      } else if ( type == float.class ) {
         return  new Float (string);
      } else if ( type == double.class ) {
         return  new Double (string);
      } else if ( type == short.class ) {
         return  new Short (string);
      } else if ( type == byte.class ) {
         return  new Byte (string);
      } else if ( type == char.class ) {
         return  new Character ( string.charAt(0) );
      } else {
         throw new Error("Unhandled type in HTMLChoiceFieldEditor : "+type);
      }
   }

}
