/*

  Copyright (C) 2001 Laurent Martelli
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA */

package jac.aspects.gui;

import java.util.Collection;
import java.util.Iterator;

import jac.core.ObjectRepository;
import jac.core.NameRepository;

import jac.core.rtti.ClassItem;
import jac.core.rtti.FieldItem;
import jac.util.Predicate;
import jac.util.Log;

/**
 * This is a special value editor that allows the user to choose a
 * value within a set of object of a given type. */

public class HTMLObjectChooser extends AbstractHTMLEditor implements HTMLEditor 
{

   ClassItem type;
   FieldItem reference;
   Predicate predicate;
   boolean allowNull;
   boolean allowNew;

   /**
    * Constructs a new object chooser.
    *
    * @param type the type of the objects that will automatically fill
    * the choices presented to the user
    * @param reference the subtance reference that is affected by this
    * chooser (can be null) 
    * @param predicate if not null, further specify which objects to chose from
    */

   public HTMLObjectChooser(ClassItem type, FieldItem reference, Predicate predicate,
                            boolean allowNull, boolean allowNew) 
   {
      this.type = type;
      this.reference = reference;
      this.predicate = predicate;
      this.allowNew = allowNew;
      this.allowNull = allowNull;
   }

   /*
    * HTMLEditor interface
    */

   public String genHTML(String name) { 
      Log.trace("gui.chooser","HTMLObjectChooser.type = "+type+" (value="+value+")");

      String html = "<SELECT NAME="+name+">\n";
      Collection all_objects = ObjectRepository.getObjects(type);
      if (allowNull)
         all_objects.add(null);

      String selected;
      Iterator i = all_objects.iterator();
      while(i.hasNext()) {
         Object curobj = i.next();
         if (predicate==null || predicate.apply(curobj)) {
            selected = ( curobj == value ? " selected" : "" );
            if (curobj==null) 
               html += "<OPTION VALUE=\"\""+selected+"></OPTION>\n";
            else
               html += "<OPTION VALUE=\""+NameRepository.get().getName(curobj)+
                  "\""+selected+">"+curobj.toString()+"</OPTION>\n";
         }
      }
      html += "</SELECT>\n";

      if( value != null ) {
         html += "<input class=method type=submit "+
            "name=\"_VIEWREF_"+name+"\" "+
            "value=\"view\"> ";
      }
      if (allowNew) {
         html += "<input class=method type=submit name=\"_NEW_"+type.getName()+"\" "+
            "value=\"new\"> ";
      }
      return html;
   }

   public Object getValue(String string) {
      return NameRepository.get().getObject( string );
   }
} 
