/*

  Copyright (C) 2001 Laurent Martelli
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA */

package jac.aspects.gui;

/**
 * HTML editor for primitive types (int, float, long, double, short,
 * byte, char), String, and other types whose constructor takes a
 * String.
 */

public class HTMLPrimitiveFieldEditor extends AbstractHTMLEditor 
   implements HTMLEditor, HTMLViewer
{
   boolean isPassword;

   public HTMLPrimitiveFieldEditor() {
      this.isPassword = false;
   }

   public HTMLPrimitiveFieldEditor(boolean isPassword) {
      this.isPassword = isPassword;
   }

   public String genHTML(String name) 
   {
      return "<INPUT type="+(isPassword?"PASSWORD":"TEXT")+
         " class=editor"+
         " name=\""+name+"\" "+
         "value=\""+ ( value==null ? "" : value.toString() ) +"\">";
   }

   public String genHTML(Object value, boolean isSmall)
   {
      return value.toString();
   }

   public Object getValue(String string) {
      if ( type == int.class ) {
         return  new Integer (string);
      } else if ( type == boolean.class ) {
         return  new Boolean (string);
      } else if ( type == long.class ) {
         return  new Long (string);
      } else if ( type == float.class ) {
         return  new Float (string);
      } else if ( type == double.class ) {
         return  new Double (string);
      } else if ( type == short.class ) {
         return  new Short (string);
      } else if ( type == byte.class ) {
         return  new Byte (string);
      } else if ( type == char.class ) {
         return  new Character ( string.charAt(0) );
      } else {
         try {
            return type.getConstructor(new Class[] {String.class}).
               newInstance(new Object[] {string});
         } catch (Exception e) {
            throw new Error(e.toString());
         }
      }
   }
}
