/*

  Copyright (C) 2001 Renaud Pawlak, Laurent Martelli
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA */

package jac.aspects.gui;

import java.text.*;
import java.net.URL;
import java.awt.Component;
import java.awt.Color;
import java.awt.Image;
import javax.swing.*;
import javax.swing.table.TableCellRenderer;

/**
 * A Swing viewer component for date values.
 */

public class ImageURLViewer extends JLabel implements ValueViewer, TableCellRenderer
{
   
   /**
    * Constructs a new date editor. */

   public ImageURLViewer() {
      super();
   }

   /**
    * Sets the value of the edited date.
    *
    * @param value a <code>Date</code> instance */

   public void setValue(Object value) {
      if (value!=null)
         setIcon(new ImageIcon((URL)value));
      else
         setIcon(null);
   }

   // TableCellRenderer
   public Component getTableCellRendererComponent(JTable table, Object value, 
                                                  boolean isSelected, boolean hasFocus, 
                                                  int row, int column) 
   {
      setOpaque(true); // so that the background is really drawn
      if (isSelected) {
         setForeground(table.getSelectionForeground());
         setBackground(table.getSelectionBackground());
      } else {
         setForeground(table.getForeground());
         setBackground(table.getBackground());
      }

      setValue(value);
      if (value!=null) {
         // scale down the image to fit into the cell
         ImageIcon icon = (ImageIcon)getIcon();
         icon.setImage(icon.getImage().getScaledInstance(100,20,Image.SCALE_SMOOTH));
      }

      return this;
   }
}
