/*
  Copyright (C) 2001 Renaud Pawlak, Laurent Martelli
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA */

package jac.aspects.gui;

import jac.util.*;
import jac.core.*;
import jac.core.rtti.*;
import java.lang.reflect.*;
import java.util.*;
import java.awt.*;

/**
 * This wrapper asks the user the parameters of the invoked method if
 * the attribute <code>Gui.askForParameters</code> is defined in the
 * current collaboration.
 *
 * @see jac.core.Display#showInput(Object,AbstractMethodItem,Object[])
 * @see InputSequence
 * @see InputBox */

public class InputWrapper extends Wrapper {

   boolean areAllWrappees(Class[] types) {
      for(int i=0;i<types.length;i++) {
         if( ! Wrappee.class.isAssignableFrom(types[i]) ) {
            return false;
         }
      }
      return true;
   }
  
   /**
    * Calls the <code>Display.showInput</code> method on the current
    * display.
    *
    * @return the value returned by the wrapped method
    * @see jac.core.Display#showInput(Object,AbstractMethodItem,Object[]) */

   public Object askForParameters() throws InputFailedException, InputCanceledException {

      boolean proceed = true;

      // DIRTY HACK
      ClassItem cli = ClassRepository.get().getClass( wrappee().getClass() );
      MethodItem[] methods = cli.getMethods( method() );
      MethodItem method = null;
      for (int i=0; i<methods.length; i++) {
         if (methods[i].getParameterTypes().length == args().length) {
            Log.trace("gui.inputWrapper",2,"found method "+methods[i]);
            method = methods[i];
            break;
         } else {
            Log.trace("gui.inputWrapper",2,"skipping method "+methods[i]);
         }
      }
      if (method==null) {
         throw new InputFailedException(
            "Could not find a matching method for "+method());
      }
      Display display = (Display)attr("Gui.display");
      
      if( attr( "Gui.askForParameters" ) == method ) {
         Log.trace("gui.inputWrapper","Entering parameters asking for " + wrappee() + " : "+ 
                   method()+" askForParameters = " + attr( "Gui.askForParameters" )+
                   " autoCreate="+attr("Gui.autoCreate")+" view="+attr("Gui.view") );


         lattrdef( "Gui.askForParameters", null );

         if ( display!=null ) {

            Class[] parameters = method.getParameterTypes();
            // automatic created parameter handling
            if( (method.getAttribute("Gui.autoCreate") != null || 
                attr( "Gui.autoCreate" ) != null) && areAllWrappees(parameters) ) 
            {
               // freeze the GC to avoid the deletion of the temporary objects
               GarbageCollector.freeze();
               Log.trace("gui.inputWrapper", "in parameter autoCreation" );
               attrdef("Gui.autoCreation","true");
               boolean ok=true;
               for(int i=0; i<parameters.length; i++) {
                  //if( parameters[i] instanceof Wrappee ) {
                  try {
                     args()[i]=parameters[i].newInstance();
                  } catch( Exception e ) {
                     e.printStackTrace();
                  }
                  ok = display.showModal(args()[i],
                                         "Creation of a new "+
                                         NamingConventions.getShortClassName(
                                            args()[i].getClass()),
                                         "Fill the needed information and validate.",
                                         true, true, false);
                  if(!ok) {
                     System.out.println("****** INPUT CANCELLED ****");
                     proceed = false;
                     break;
                  } else {
                     if (attr("Gui.view")!=null) {
                        ((Wrappee)args()[i]).invokeRoleMethod("addView", new Object[] {attr("Gui.view")});
                     }
                     // if some field must be autocreated, we ask the user
                     // to input them (except if already filled)
                     // this should be reccursive but it is not implemented
                     // for the moment
                     ClassItem parameterClass = ClassRepository.get()
                        .getClass(parameters[i]);
                     String[] autoCreatedFields = 
                        (String[])parameterClass
                        .getAttribute("Gui.autoCreatedState");
                     if( autoCreatedFields != null ) 
                     {
                        for( int j=0; j<autoCreatedFields.length; j++ ) 
                        {
                           FieldItem f = parameterClass
                              .getField(autoCreatedFields[j]);
                           Object newInstance=null;
                           if( f!= null &&
                               f.getThroughAccessor(args()[i]) == null ) 
                           {
                              try {
                                 newInstance = f.getType().newInstance();
                              } catch( Exception e ) {
                                 e.printStackTrace();
                              }
                              ok = display.showModal(newInstance, 
                                                     "Creation of a new "+
                                                     NamingConventions.getShortClassName(
                                                        newInstance.getClass()),
                                                     "You did not fill any "+f.getName()+". "+
                                                     "Fill the needed informations and validate.",
                                                     true, true, false);
                              if(ok) {
                                 f.setThroughWriter(args()[i],newInstance);
                              }
                           }
                        }
                     }
                  }
               }
               GarbageCollector.unfreeze();
               attrdef("Gui.autoCreation",null);
            } else {

               Class[] pts = method.getParameterTypes();
               Object[] askingSequence = (Object[])method.getAttribute("Gui.askingSequence");
               boolean allCreated = false;
               if( pts.length > 0 ) {
                  Log.trace("gui.inputWrapper","5");
                  if( askingSequence != null ) {
                     Log.trace("gui.inputWrapper","input sequence found");
                     allCreated = true;
                     for( int i=0; i<pts.length; i++ ) {
                        if( askingSequence[i].equals("autoCreate") ) {
                           args()[i] = create( 
                              ClassRepository.get().getClass(pts[i]), display );
                        } else {
                           allCreated = false;
                        }
                     }
                  }
                  if( ! allCreated ) {
                     Log.trace("gui.inputWrapper","asking for parameters");
                     proceed = display.showInput( wrappee(), method, args() );
                     Log.trace("gui.inputWrapper","args="+Arrays.asList(args()));
                  } else {
                     Log.trace("gui.inputWrapper","skipping asking for parameters");
                     proceed = true;
                  }
               }
            }
         } else {
            Log.trace("gui","No display available");
            throw new InputFailedException("No display available");
         }
         Log.trace("gui.inputWrapper","proceed = "+proceed);
         
      }
      
      if( proceed ) {
         return proceed();
      } else {
         GarbageCollector.unfreeze();
         throw new InputCanceledException(method());
      }
      
   }

   /**
    * This method performs all the inputs operations for an instance creation.
    *
    * @param classItem the class to instantiate
    * @param display the display to use */

   public Object create(ClassItem classItem, Display display) 
      throws InputFailedException, InputCanceledException {

      Object newInstance = null;
      Vector constructors = classItem.getConstructors();
      ConstructorItem c;
      if (constructors.size()==1) {
         c = (ConstructorItem)classItem.getConstructors().get(0);
      } else {
         c = (ConstructorItem)classItem.getConstructors().get(1);
      }
      if ( c.getParameterTypes().length > 0 ) {
         Object[] parameters = new Object[c.getParameterTypes().length];
         
         String inputSequenceName =  (String) c.getAttribute("Gui.inputSequence");
         Log.trace("webgui","input sequence name = "+inputSequenceName );
         if( inputSequenceName == null ) {
            display.showInput ( null, c, parameters );
         } else {
            jac.aspects.gui.InputSequence inputSeq = null;
            try {
               inputSeq = (jac.aspects.gui.InputSequence) Class.forName( inputSequenceName )
                  .getConstructor(new Class[] { Display.class, AbstractMethodItem.class, Object[].class })
                  .newInstance(new Object[] {display, c, parameters});
            } catch ( Exception e ) {
               throw new InputFailedException( "unabled to create input sequence" );
            }
            if( ! inputSeq.proceedInputs() ) {
               throw new InputCanceledException("input sequence was canceled");
            }
         }
         
         try {
            newInstance = c.newInstance(parameters);
            ((Wrappee)newInstance).forceTreatment();
         } catch ( Exception e ) {
            e.printStackTrace();
         }
         
      } else {
         try {
            newInstance = c.newInstance();
            // This is a HACK because the persistence aspect may have
            // unnamed the object
            ((Wrappee)newInstance).forceTreatment();
         } catch ( Exception e ) {
            e.printStackTrace();
         }
      }
      return newInstance;
   }

   public void catchInputCanceled(InputCanceledException e) {
      Log.trace("gui.inputWrapper",e.toString());
      attrdef("Gui.autoCreation",null);
   }
}


class InputFailedException extends Exception {
    public InputFailedException() {}
    public InputFailedException(String msg) {super(msg);}
}
