/*
  Copyright (C) 2001 Renaud Pawlak, Laurent Martelli
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA */

package jac.aspects.gui;

import java.text.*;
import java.util.*;
import java.awt.Dimension;
import java.awt.Component;
import javax.swing.*;
import java.lang.reflect.Constructor;

import jac.util.Log;

/**
 * A Swing editor component for fields values (primitive types).
 */

public class PrimitiveFieldEditor extends JPanel implements ValueEditor {
   
   Class type;
   JTextField textField;

   /**
    * Constructs a new primitive field editor. */
   
   public PrimitiveFieldEditor(Class type) {
      this(type,false);
  }

   /**
    * Constructs a new primitive field editor. */

   public PrimitiveFieldEditor(Class type, boolean password) {
      this.type = type;
      if (password) 
         textField = new JPasswordField(20);
      else  
         textField = new JTextField(20);
      add(textField);
  }

   /**
    * Gets the value of the edited field.
    *
    * @return an object for the value */
 
   public Object getValue() {
      if ( type == int.class || type == Integer.class ) {
         return( new Integer (textField.getText()) );
      } else if ( type == boolean.class || type == Boolean.class) {
         return( new Boolean (textField.getText()) );
      } else if ( type == long.class || type == Long.class ) {
            return( new Long (textField.getText()) );
      } else if ( type == float.class || type == Float.class ) {
         return( new Float (textField.getText()) );
      } else if ( type == double.class || type == Double.class ) {
         return( new Double (textField.getText()) );
      } else if ( type == short.class || type == Short.class ) {
         return( new Short (textField.getText()) );
      } else if ( type == byte.class || type == Byte.class ) {
         return( new Byte (textField.getText()) );
      } else if ( type == char.class || type == Character.class ) {
         return( new Character ( textField.getText().charAt(0) ) );
      } else if ( type == String.class ) {
         return( textField.getText() );
      } else {
         try {
            // trying to construct the object from its textual 
            // representation (I think that any class should have
            // a constructor taking a string... this is so helpful...)
            // of course, this will raise an exception most of the time :-(
            Constructor c = type.getConstructor(new Class[] {String.class});
            return c.newInstance( new Object[] {textField.getText()} );
         } catch( Exception e ) {
            e.printStackTrace();
            throw new RuntimeException("Unhandled type "+type.getName());
         }
      }
   }

   /**
    * Sets the value of the edited field.
    *
    * @param value a value */

   public void setValue(Object value) {
      if( value == null ) value="";
      textField.setText(value.toString());
   }

   public void setWidth(int width) {
      textField.setColumns(width);
   }

   public void setHeight(int height) {}

   /**
    * Set the focus on the textField
    */
   public void requestFocus() {
      textField.requestFocus();
      Log.trace("gui.focus","focusing "+textField.getClass().getName());
   }

   public Component getValueComponent() {
      return textField;
   }
   public void onSetFocus(Object extraOption) {}
}
