/*
  Copyright (C) 2001 Renaud Pawlak, Laurent Martelli
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA */

package jac.aspects.gui;

import jac.core.Jac;
import jac.util.Log;
import java.net.URL;
import java.util.Hashtable;
import javax.swing.ImageIcon;


/**
 * A class with static methods to get GUI resources as icons.<p>
 *
 * @author <a href="mailto:renaud@cnam.fr">Renaud Pawlak</a>
 * @author <a href="mailto:laurent@aopsys.com">Laurent Martelli</a>
 */

public class ResourceManager {
   
   /** Store the path where to find resources for the GUI. */ 
   public static String RESOURCES_PATH = "jac/aspects/gui/resources/"; 
   //Jac.getJacRoot() + "/src/jac/aspects/gui/resources/";

   static Hashtable icons = new Hashtable();
   
   /** 
    * Creates an incon from a path string.
    *
    * @param path the path of the icon (absolute or accessible through
    * the classpath) */

   public static ImageIcon createIcon(String path) {
      Log.trace("resources","resolving URL for "+path);
      URL url = ClassLoader.getSystemClassLoader().getResource(path);
      Log.trace("resources","URL="+url);
      return new ImageIcon(url);
   }

   /**
    * This method allows the user to define a new icon resource from
    * its file name.
    *
    * <p>The used name to identify the resource can be used in
    * <code>getIcon</code>.
    *
    * @param name the resource name
    * @param path the resource path
    * @see #getIcon(String) */
 
   public static void setIcon(String name, String path) {
      icons.put(name,createIcon(path));
   }

   /**
    * Retrieves an icon previously defined by <code>setIcon</code>.
    *
    * @param name the resource name
    * @see #setIcon(String,String) */

   public static ImageIcon getIcon(String name) {
      return (ImageIcon)icons.get(name);
   }

   // BUILT-IN RESOURCES

   /** The edit icon. */
   private static ImageIcon editIcon = null;

   /**
    * Gets the edit icon resource.
    * @return the icon */

   public static ImageIcon getEditIcon() {
      if ( editIcon == null )
         editIcon = createIcon(RESOURCES_PATH+"small_edit.gif");
      return editIcon;
   }

   /** The view icon. */
   private static ImageIcon viewIcon = null;

   /**
    * Gets the view icon resource.
    * @return the icon */

   public static ImageIcon getViewIcon() {
      if ( viewIcon == null )
         viewIcon = createIcon(RESOURCES_PATH+"small_view.gif");
      Log.trace("resources","viewIcon="+viewIcon);
      return viewIcon;
   }

   /** The new icon. */
   private static ImageIcon newIcon = null;

   /**
    * Gets the new icon resource.
    * @return the icon */

   public static ImageIcon getNewIcon() {
      if ( newIcon == null ) 
         newIcon = createIcon(RESOURCES_PATH+"small_new.gif");
      Log.trace("resources","newIcon="+newIcon);
      return newIcon;
   }

   /** The remove icon. */
   private static ImageIcon removeIcon = null;

   /**
    * Gets the remove icon resource.
    * @return the icon */

   public static ImageIcon getRemoveIcon() {
      if ( removeIcon == null )
         removeIcon = createIcon(RESOURCES_PATH+"small_remove.gif");
      return removeIcon;
   }

   /** The code icon. */
   private static ImageIcon codeIcon = null;

   /**
    * Gets the code icon resource.
    * @return the icon */

   public static ImageIcon getCodeIcon() {
      if ( codeIcon == null )
         codeIcon = createIcon(RESOURCES_PATH+"small_code.gif");
      return codeIcon;
   }

   /** The information icon. */
   private static ImageIcon informationIcon = null;

   /**
    * Gets the information icon resource.
    * @return the icon */

   public static ImageIcon getInformationIcon() {
      if ( informationIcon == null )
         informationIcon = createIcon(RESOURCES_PATH+"small_information.gif");
      return informationIcon;
   }   


   /** The group icon. */
   private static ImageIcon groupIcon = null;

   /**
    * Gets the group icon resource.
    * @return the icon */

   public static ImageIcon getGroupIcon() {
      if ( groupIcon == null )
         groupIcon = createIcon(RESOURCES_PATH+"group.gif");
      return groupIcon;
   }   

}
