/*

  Copyright (C) 2001 Renaud Pawlak, Laurent Martelli
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA */

package jac.aspects.gui;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.*;
import java.lang.reflect.*;
import jac.core.*;
import jac.util.*;
import jac.aspects.naming.*;

/**
 * This dialog is used to show the result of a called method to the
 * user.<p>
 *
 * @see View
 * @see CallingBox */

public class ResultBox extends JDialog implements ActionListener {

   private JButton okButton;

   private Object substance = null;
   private Method method = null;

   /**
    * Construct a new result box.<p>
    *
    * @param parent the parent dialog (a view or a calling box)
    * @param substance the object that furnished the result
    * @param method the method that was called
    * @param result the result of the called method */
   
   public ResultBox( Dialog parent, Object substance, Method method, Object result ) {
      super( parent, "Result for " + method.getName() + " on " + 
             NameRepository.get().getName( substance ), true );
      setModal( true );
      this.substance = substance;
      this.method = method;

      Container contentPane = getContentPane();

      Class substance_type = substance.getClass();

      String tn = substance_type.getName();

      addWindowListener ( new WindowAdapter () {
            public void windowClosing( WindowEvent e ) {
               e.getWindow().dispose();
            }
         }
      );
      
      JPanel panel = new JPanel();
      Vector containers = new Vector();
      Component cur_component = null;
      Component extra_component = null;
      Component super_component = null;
      panel.setBorder( 
         BorderFactory.createTitledBorder( 
            BorderFactory.createEtchedBorder(),
            " Result " ) );
      panel.setLayout( new BoxLayout( panel, BoxLayout.Y_AXIS ) );

      Collection cvalue = null;
      String[] names = null;

      if ( result.getClass().isArray() ) {
         cvalue = Arrays.asList( (Object[])result );
      }

      if ( result instanceof Collection ) {
         cvalue = (Collection)result;
      }

      if ( result instanceof Map ) {
         cvalue = ((Map)result).values();
      }

      if ( cvalue != null ) {
      
         names = new String[cvalue.size()];
         Iterator it = cvalue.iterator();
         for ( int j = 0; j < names.length; j++ ) {
            try {
               Object cur = it.next();
               if ( cur != null ) {
                  if ( (names[j] = NameRepository.get().getName( cur )) == null ) {
                     names[j] = cur.toString();
                  }
               } else {
                  names[j] = "null";
               }
            } catch (Exception e) { 
               names[j] = "null"; 
            }
         }
      }

      if ( cvalue != null ) {
         
         JList wordList = new JList( names );
         JScrollPane scrollPane = new JScrollPane(wordList);
         
         //wordList.setName( fields[i].getName() );
         //state_components.add( wordList );
         
         //wordList.setVisibleRowCount( 4 );
         // p = new JPanel();
         //container.add( scrollPane );
         //wordList.addListSelectionListener(this);
         
         panel.add( scrollPane );
         
      } else if (result.getClass() == java.util.Date.class) {
         SimpleDateFormat format = new SimpleDateFormat("dd/MM/yyyy");
         panel.add( new JLabel(format.format((Date)result)) );
      } else {
         panel.add( new JLabel( result.toString() ) );
      
      }
         
      contentPane.add("Center", panel);
      JPanel p2 = new JPanel();
      p2.setBorder( BorderFactory.createEtchedBorder() );
      okButton = addButton( p2, "Ok" );
      contentPane.add( "South", p2 );
      pack();
      setVisible ( true );
   }

   /**
    * Internally used. <p> */

   JButton addButton(Container c, String name)  {
      JButton button = new JButton(name);
      button.addActionListener(this);
      c.add(button);
      return button;
   }

   /**
    * Close the dialog when <code>evt</code> corresponds to the OK
    * button.<p> */
    
   public void actionPerformed(ActionEvent evt) {

      try {

         Object source = evt.getSource();
         if(source == okButton) {
            setVisible(false);
         }
      } catch ( Exception e ) {
         e.printStackTrace ();
      }
   }
   
}











