/*
  Copyright (C) 2001 Renaud Pawlak
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA */

package jac.aspects.gui;

import java.text.*;
import java.util.*;
import javax.swing.*;
import java.awt.Dimension;
import java.lang.reflect.Array;
import java.awt.Component;

/**
 * A Swing editor component for multi-lines text values.
 */

public class TextEditor extends JPanel implements ValueEditor {

   JEditorPane editor;
   JScrollPane scrollPane;
   Class type;
   
   int defaultWidth = 400;
   int defaultHeight = 400;

   /**
    * Constructs a new text editor. */

   public TextEditor(Class type) {
      setLayout(new BoxLayout(this,BoxLayout.Y_AXIS));
      add(scrollPane=new JScrollPane(editor=new JEditorPane()));
      scrollPane.setPreferredSize(new Dimension( defaultWidth, defaultHeight ));
      //.setPreferredSize(new Dimension( defaultWidth, defaultHeight ));
      this.type = type;
      editor.setContentType("html");
   }

   /**
    * Gets the value of the edited text.
    *
    * @return a string object */
 
   public Object getValue() {
      return editor.getText();
   }

   /**
    * Sets the value of the edited array
    **/

   public void setValue(Object value) {
      if( value == null ) editor.setText("");
      else editor.setText((String)value);
   }

   public void setWidth(int width) {
      defaultWidth = width;
      scrollPane.setPreferredSize(new Dimension( defaultWidth, defaultHeight ));
   }

   public void setHeight(int height) {
      defaultHeight = height;
      scrollPane.setPreferredSize(new Dimension( defaultWidth, defaultHeight ));
   }

   public Component getValueComponent() {
      return editor;
   }


   public void onSetFocus(Object extraOption) {
      jac.util.Log.trace("gui.events","TextEditor.onSetFocus "+extraOption);
      if (extraOption!=null && extraOption instanceof Integer) {
         int line = ((Integer)extraOption).intValue();
         String text = editor.getText();
         int index = 0;
         int previndex = 0;
         for (int i=0; i<line; i++) {
            previndex = index;
            index = text.indexOf('\n',index+1);
            jac.util.Log.trace("gui.events","TextEditor.onSetFocus "+previndex+","+index);
         }
         jac.util.Log.trace("gui.events","TextEditor.onSetFocus "+previndex+","+index);
         editor.setSelectionStart(previndex+1);
         editor.setSelectionEnd(index);
      }
   }

   /**
    * Set the focus on the TextEditor
    */
   public void requestFocus() {
      editor.requestFocus();
      jac.util.Log.trace("gui.focus","focusing "+editor.getClass().getName());
   }
   
}
