/*
  Copyright (C) 2002 Renaud Pawlak
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA */

package jac.aspects.gui;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

import jac.core.rtti.*;
import jac.util.*;

import jac.core.*;
import jac.wrappers.*;
import jac.aspects.naming.*;

/**
 * This class defines a Swing component tree view for objects that are
 * related to a root object through relations or collections.
 *
 * @see GuiAC */

public class TreeView extends JPanel
   implements View, ActionListener, TreeSelectionListener, MouseListener,
              TreeExpansionListener {

   JTree tree;
   Window parent;
   Object substance = null;
   private View parentView = null;
   String pathDef = null;
   boolean showRelations = true;

   JButton viewButton = null;
   JButton newButton = null;
   JButton removeButton = null;

   HashSet expandedObjects = new HashSet();

   boolean expanding = false;

   DefaultMutableTreeNode rootNode = null;

   /**
    * Constructs a new tree view.
    *
    * @param parent the Swing dialog parent
    * @param parentView a view that aggregates this view if any
    * @param substance the object the viewed collection belongs to
    * @param collection the collection that is showed by this view
    * @param showRelations if false, the relations are not showed as
    * tree elements */

   public TreeView(Window parent, 
                   View parentView, Object substance,
                   String pathDef, boolean showRelations ) {
      this.parent = parent;
      this.substance = substance;
      this.parentView = parentView;
      this.pathDef = pathDef;
      this.showRelations = showRelations;

      setLayout( new BoxLayout( this, BoxLayout.Y_AXIS ) );

      JScrollPane upperCont = new JScrollPane(tree=new JTree());
      tree.addTreeExpansionListener(this);
      //upperCont.setLayout( new BoxLayout( upperCont, BoxLayout.Y_AXIS ) );
      JPanel downCont = new JPanel();
      //downCont.setLayout( new BoxLayout( downCont, BoxLayout.X_AXIS ) );

      tree.putClientProperty("JTree.lineStyle", "Angled");
      ToolTipManager.sharedInstance().registerComponent( tree );
      tree.setRootVisible(false);
      tree.setCellRenderer( new TreeNodeRenderer() );
      tree.addTreeSelectionListener( this );
      
      createTree();

      //upperCont.add(tree);

      // add, remove and view buttons
      JButton b = null;
      b = viewButton = new JButton ( ResourceManager.getViewIcon() );
      b.setToolTipText( "View" );
      b.setActionCommand( "open" );
      b.addActionListener( this );
      downCont.add( b );

      b = newButton = new JButton ( ResourceManager.getNewIcon() );
      b.setToolTipText( "Add" );
      b.setActionCommand( "add" );
      b.addActionListener( this );
      downCont.add( b );

      b = removeButton = new JButton ( ResourceManager.getRemoveIcon() );
      b.setToolTipText( "Remove" );
      b.setActionCommand( "remove" );
      b.addActionListener( this );
      downCont.add( b );

      add( "North", upperCont );
      add( "South", downCont );

   }

   void addNodes( DefaultMutableTreeNode rootNode, 
                  Collection accessibles, String curPathDef ) {
      String rExpr = null;
      String oExpr = null;
      Log.trace("treeview","add nodes with accessibles="+accessibles+
                " curPathDef="+curPathDef);
      int i = curPathDef.indexOf("/"); 
      int i2=0;
      boolean end = false;
      if( i != -1 ) {
         rExpr = curPathDef.substring(0,i);
         i2 = curPathDef.indexOf("/",i+1);
         if( i2 == -1 ) {
            oExpr = curPathDef.substring(i+1);
         } else {
            oExpr = curPathDef.substring(i+1,i2);
         }
      } else end=true;
      Log.trace("treeview","rExpr="+rExpr+", oExpr="+oExpr+", end="+end);
      ClassRepository cr = ClassRepository.get();
      // iterates on all the objects
      Iterator it = accessibles.iterator();
      while( it.hasNext() ) {
         Object o = it.next();
         if(o==null) continue;
         if( o instanceof Wrappee ) {
            // forces the treatment of the wrappe
            //((Wrappee)o).forceTreatment();
            // to refresh tree
            ((Wrappee)o).invokeRoleMethod("addView", new Object[] {this});
         }
         // add current as a node
         DefaultMutableTreeNode n = new DefaultMutableTreeNode(o); 
         rootNode.add( n );

         if( !end ) {
            // iterates on all the relations
            ClassItem cli = cr.getClass( o.getClass() );
            Collection rels = cli.getMatchingRelations( rExpr );
            Collection related=null;

            Iterator itRel = rels.iterator();
            while( itRel.hasNext() ) {
               Object rel = itRel.next();
               if(!GuiAC.isVisible((MetaItem)rel)) continue;
               if( rel instanceof CollectionItem ) {
                  DefaultMutableTreeNode nRel = null;
                  if( showRelations ) {
                     // add a collection node
                     nRel = new DefaultMutableTreeNode(rel); 
                     n.add( nRel );
                     // to refresh tree
                     if(rel instanceof Wrappee) {
                        ((Wrappee)rel).invokeRoleMethod("addView", new Object[] {this});
                     }
                  }
                  Collection cRel = ((CollectionItem)rel).getActualCollection(o);
                  related = MethodPointcut.filterObjects( cRel, oExpr );
                  // reccursively add nodes
                  if( showRelations ) { 
                     addNodes(nRel,related,curPathDef.substring(i2+1));
                  } else {
                     addNodes(n,related,curPathDef.substring(i2+1));
                  }
               }/* else if( rel instanceof FieldItem ) {
                  Object relatedObject = ((FieldItem)rel).get(o);
                  related = new Vector();
                  related.add(relatedObject);
                  addNodes(n,related,curPathDef.substring(i2+1));
                  }*/
            }
         }
      }
   }

   /**
    * Creates the tree from its internal definition.
    *
    * @see #setPathDef(String) */

   public void createTree() {
      
      expanding=true;

      rootNode = new DefaultMutableTreeNode();

      Log.trace("treeview","creating tree "+pathDef);
            
      int i = pathDef.indexOf("/"); 
      String init;

      if( i == -1 ) {
         init = pathDef;
      } else {
         init = pathDef.substring(0,i);
      }

      // the first accessible collection
      Collection accessibles = ObjectRepository.getObjects( init );

      addNodes( rootNode, accessibles, pathDef.substring(i+1) );
            
      DefaultTreeModel newTreeModel = new DefaultTreeModel( rootNode );

      tree.invalidate();
      tree.setModel( newTreeModel );

      tree.addMouseListener(this);
      expandNodes(rootNode);
      tree.validate();
      expanding = false;
   }

   void expandNodes(DefaultMutableTreeNode root) {
      Enumeration childs=root.children();
      while(childs.hasMoreElements()) {
         DefaultMutableTreeNode node = (DefaultMutableTreeNode)childs.nextElement();
         if( expandedObjects.contains(node.getUserObject()) ) {
            tree.expandPath(new TreePath(node.getPath()));
            expandNodes(node);
         }
      }
   }

   void selectObject(DefaultMutableTreeNode root,Object toSelect) {
      Enumeration childs=root.children();
      System.out.println("selecting object "+toSelect+" in "+childs);
      while(childs.hasMoreElements()) {
         DefaultMutableTreeNode node = (DefaultMutableTreeNode)childs.nextElement();
         if( node.getUserObject()==toSelect ) {
            System.out.println("found object");
            DefaultMutableTreeNode parent = (DefaultMutableTreeNode)node.getParent();
            while (parent!=null) {
               tree.expandPath(new TreePath(parent.getPath()));
               parent = (DefaultMutableTreeNode)parent.getParent();
            }
            tree.expandPath(new TreePath(node.getPath()));
            tree.addSelectionPath(new TreePath(node.getPath()));
         }
         selectObject(node,toSelect);
      }
   }

   public void treeCollapsed(TreeExpansionEvent event) {
      if( expanding ) return;
      expandedObjects.remove(((DefaultMutableTreeNode)event.getPath()
                              .getLastPathComponent()).getUserObject());
   }

   public void treeExpanded(TreeExpansionEvent event) {
      if( expanding ) return;
      expandedObjects.add(((DefaultMutableTreeNode)event.getPath()
                           .getLastPathComponent()).getUserObject());
   }

   /**
    * Sets a new definition to the tree.
    *
    * @param pathDef the object-path definition
    *
    * @see #createTree() */

   public void setPathDef(String pathDef) {
      this.pathDef = pathDef;
   }

   /**
    * Handles the tree view selection changes. */

   public void valueChanged(TreeSelectionEvent event) {
      DefaultMutableTreeNode node = (DefaultMutableTreeNode)
                           tree.getLastSelectedPathComponent();        
      if (node == null) return;
      Object selected = node.getUserObject();
      if( selected instanceof CollectionItem ) {
         viewButton.setEnabled(false);
         newButton.setEnabled(true);
         removeButton.setEnabled(true);
      } else if ( selected instanceof Wrappee ) {
         viewButton.setEnabled(true);
         newButton.setEnabled(false);
         removeButton.setEnabled(true);
         fireOnSelectionEvents(node);
      }
   }

   void fireOnSelectionEvents(DefaultMutableTreeNode node) {
      // fire the onSelection event
      DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode)node.getParent();
      if( parentNode != null ) { 
         Object c = parentNode.getUserObject();
         if( c instanceof CollectionItem || !showRelations ) {            
            DefaultMutableTreeNode gparentNode = parentNode;
            if( !showRelations ) {
               c = getFieldForObject(c,node.getUserObject());
            } else {
               gparentNode = (DefaultMutableTreeNode)parentNode.getParent();
            }
            // recursively send the event for the parent nodes
            if( gparentNode != null )
               fireOnSelectionEvents(gparentNode);
            if( c != null ) {
               // send it for the current node
               ((CustomizedGUI)parent).onSelection(
                  (CollectionItem)c,node.getUserObject(),null,null);
            }
         }
      }
   }

   FieldItem getFieldForObject(Object container, Object contained) {
      if( container == null ) return null;
      ClassRepository cr = ClassRepository.get();
      ClassItem cli = cr.getClass( container.getClass() );
      CollectionItem[] colls = cli.getCollections();
      for(int i=0;i<colls.length;i++) {
         if( colls[i].getActualCollection(container).contains(contained) )
            return colls[i];
      }
      return null;
   }
   
   /**
    * Handles the actions on this view.
    *
    * <p>On a collection view, the three default possible actions are
    * to open a new view on the selected item, to add a new item to
    * the collection, and to remove the selected item from the
    * collection.
    *
    * @param evt the user event */

   public void actionPerformed(ActionEvent evt) {

      String arg = evt.getActionCommand();
      Log.trace("treeview","action performed: "+arg);
      DefaultMutableTreeNode node = (DefaultMutableTreeNode)tree
         .getAnchorSelectionPath().getLastPathComponent();
      DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode)node.getParent();

      JacObject.getCollaboration().addTransientLocalAttribute(
         "Gui.display", parent);
      JacObject.getCollaboration().addPersistentAttribute(
         "Session.sid", "Swing"+jac.core.dist.Distd.getLocalContainerName() );

      if ( arg.equals( "open" ) || arg.equals( "View" )) {
         if( parentNode != null ) { 
            Object c = parentNode.getUserObject();
            if( c instanceof CollectionItem || !showRelations ) {
               
               if( !showRelations ) {
                  c = getFieldForObject(c,node.getUserObject());
               }
               if( c != null ) {
                  JPanel target = (JPanel) ((CollectionItem)c).getAttribute("Gui.Swing.targetContainer");
            
                  if( target != null ) {
                     JacObject.getCollaboration()
                        .addTransientAttribute(
                           "Gui.view.small",
                           ((CollectionItem)c).getAttribute("Gui.Swing.smallTargetContainer"));
                     Component[] comps = target.getComponents();
                     for(int i=0; i<comps.length; i++) {
                        if( comps[i] instanceof View ) {
                           ((View)comps[i]).close();
                        }
                     }
                     // removing the views
                     target.removeAll();
                     // creating and adding the new view
                     target.add( new ObjectView(parent, null, node.getUserObject()) );
                     target.validate();
                     JacObject.getCollaboration()
                        .addTransientAttribute(
                           "Gui.view.small",null);
                     return;
                  }
               }
            }
         }
         ObjectView.addViewFor( node.getUserObject(), parent, parent );
         return;
      } else if ( arg.equals( "Open view (new window)..." ) ) {

         ObjectView.addViewFor( node.getUserObject(), parent, parent );
         return;

      } else if ( arg.equals( "add" ) ) {
         Object o = node.getUserObject();
         if( o instanceof CollectionItem ) {
            if( parentNode != null ) { 
               try {
                  MethodItem[] addingMethods = ((CollectionItem)o).getAddingMethods();                  
                  if( addingMethods != null && addingMethods.length > 0 ) {
                     Log.trace("treeview","invoking "+addingMethods[0]+" on "+
                               parentNode.getUserObject());
                     if ( ((CollectionItem)o).getAttribute("Gui.autoCreate") != null ) {
                        Log.trace("treeview","auto creation asked");
                        JacObject.getCollaboration().addTransientLocalAttribute(
                           "Gui.autoCreate", addingMethods[0].getParameterTypes()[0]);
                     }
                     JacObject.getCollaboration().addTransientLocalAttribute(
                        "Gui.askForParameters", addingMethods[0]);
                     JacObject.getCollaboration().addTransientLocalAttribute(
                        "Gui.display", parent);
                     addingMethods[0].invoke(parentNode.getUserObject(),
                                             new Object[addingMethods[0].getParameterTypes().length]);
                  }
               } catch( Exception e ) {
                  e.printStackTrace();
               }
            }
         }
      } else if ( arg.equals( "remove" ) ) {
         Object o = node.getUserObject();
         if( o instanceof CollectionItem ) {
            if( parentNode != null ) { 
               try {
                  MethodItem[] removingMethods = ((CollectionItem)o).getRemovingMethods();                  
                  if( removingMethods != null && removingMethods.length > 0 ) {
                     Log.trace("treeview","invoking global "+removingMethods[0]+" on "+
                               parentNode.getUserObject());
                     JacObject.getCollaboration().addTransientLocalAttribute(
                        "Gui.askForParameters", removingMethods[0]);
                     JacObject.getCollaboration().addTransientLocalAttribute(
                        "Gui.display", parent);
                     removingMethods[0].invoke(parentNode.getUserObject(),
                                               new Object[removingMethods[0].getParameterTypes().length]);
                  }
               } catch( Exception e ) {
                  e.printStackTrace();
               }
            }
         } else {
            if( parentNode != null ) {
               Object coll = parentNode.getUserObject();
               if( coll instanceof CollectionItem ) {
                  DefaultMutableTreeNode grandParentNode = (DefaultMutableTreeNode)parentNode.getParent();
                  if( grandParentNode != null ) {
                     try {
                        MethodItem[] removingMethods = ((CollectionItem)coll).getRemovingMethods();                  
                        if( removingMethods != null && removingMethods.length > 0 ) {
                           Log.trace("treeview","invoking particular "+removingMethods[0]+" on "+
                                     grandParentNode.getUserObject());
                           JacObject.getCollaboration().addTransientLocalAttribute(
                              "Gui.askForParameters", null);
                           removingMethods[0].invoke(grandParentNode.getUserObject(),
                                                     new Object[] {o});
                        }
                     } catch( Exception e ) {
                        e.printStackTrace();
                     }
                  }
               }               
            }
         }
      } else {
      
         Object o = node.getUserObject();
         ClassItem cli = ClassRepository.get().getClass( o.getClass() );
         MethodItem method = cli.getMethod( NamingConventions.lowerFirst(
            NamingConventions.getNormalizedString(arg)) );
         JacObject.getCollaboration().addTransientLocalAttribute(
            "Gui.askForParameters", method);
         JacObject.getCollaboration().addTransientLocalAttribute(
            "Gui.display", parent);
         Log.trace("treeview","invoking "+method+" on "+o);
         //try {
         //   Object[] parameters = new Object[method.getParameterTypes().length];
         Log.trace("treeview","actual method = "+method.getActualMethod());
         ((AbstractSwingDisplay)parent).invoke(method,o,null);
         //   method.invokeWithInit(o,parameters);
         //} catch(Exception e) {
         //  e.printStackTrace();
         //}
         //View.addViewFor( actionnedObject );
         return;
      }

      /*   if (evt.getActionCommand().equals("open")) {
         Object[] to_open = getSelected();
         JPanel target = (JPanel) collection.getAttribute( "Gui.Swing.targetContainer" );
         if ( target == null ) {
            for (int i=0; i<to_open.length; i++) {
               ObjectView.addViewFor( to_open[i], parent );
            }
         } else {
            // sending the close event on the views that where opened
            // in the container
            Component[] comps = target.getComponents();
            for(int i=0; i<comps.length; i++) {
               if( comps[i] instanceof View ) {
                  ((View)comps[i]).close();
               }
            }
            // removing the views
            target.removeAll();
            // creating and adding the new view
            for (int i=0; i<to_open.length; i++) {
               target.add( new ObjectView(parent, null, to_open[i]) );
            }
            target.validate();
         }
      } else if (evt.getActionCommand().equals("add")) {
         MethodItem[] methods = collection.getAddingMethods();
         if (methods!=null && methods.length>=1) {
            MethodItem addMethod = methods[0];
            Log.trace("gui","Invoking add method "+addMethod.getName()+" on collection owner");

            if ( collection.getAttribute("Gui.autoCreate") != null ) {
               
               ClassItem type = ClassRepository.get().getClass(
                  addMethod.getParameterTypes()[0]);
               
               JacObject.getCollaboration().addTransientAttribute("Gui.askForParameters", addMethod);
               JacObject.getCollaboration().addTransientAttribute("Gui.autoCreate", type);
               JacObject.getCollaboration().addTransientAttribute("Gui.display", parent);
               //JacObject.getCollaboration().addTransientAttribute("", type);
               
               try {
                  
                  addMethod.invoke( substance, new Object[] { null } );
                  
               } catch (Exception e) {
                  //                  Log.error("Instanciation of "+type.getName()+" failed : "+e);
                  e.printStackTrace();
               }
               
            } else {
               GuiAC.invoke((Display)parent,addMethod,substance,null);
            }
         } else {
            methods = collection.getAccessingMethods();
            if (methods!=null && methods.length==1) {
               MethodItem getMethod = methods[0];
               MethodItem method = null;
               Object c;
               try {
                  c = getMethod.invoke(substance,new Object[0]);
               } catch (Exception e) {
                  Log.error("Getting collection with "+getMethod.getName()+
                            "failed : "+e);
                  return;
               }
               ClassItem cl = ClassRepository.get().getClass(c.getClass());
               if (collection.isMap()) 
                  method = cl.getMethod("put(Object,Object)");
               else
                  method = cl.getMethod("add(Object)");
               Log.trace("gui","Invoking "+method.getName()+" on collection itself");
               GuiAC.invoke((Display)parent,method,c,collection);
            }
         }
      } else if (evt.getActionCommand().equals("remove")) {
         Object[] selected = getSelected();
         MethodItem[] methods = collection.getRemovingMethods();
         if (methods!=null && methods.length>=1) {
            // try with the removing method
            MethodItem removeMethod = methods[0];            
            for (int i=0; i<selected.length; i++) {
               try {
                  Log.trace("gui","Invoking remove method "+removeMethod.getName()+" on collection owner");
                  removeMethod.invoke(substance,new Object[] {selected[i]});
               } catch (Exception e) {
                  e.printStackTrace();
               }
            }
         } else {
            // call the collection's method directly
            methods = collection.getAccessingMethods();
            if (methods!=null && methods.length==1) {
               MethodItem getMethod = methods[0];
               MethodItem method = null;
               Object c;
               try {
                  c = getMethod.invoke(substance,new Object[0]);
               } catch (Exception e) {
                  Log.error("Getting collection with "+getMethod.getName()+
                            " failed : "+e);
                  return;
               }
               ClassItem cl = ClassRepository.get().getClass(c.getClass());
               if (!collection.isMap()) {
                  method = cl.getMethod("remove(Object)");
                  Log.trace("gui","Invoking "+method.getName()+" on collection itself");
                  for (int i=0; i<selected.length; i++) {
                     try {
                        method.invoke(c,new Object[] {selected[i]});
                     } catch (Exception e) {
                        Log.error("Removing from collection with "+method.getName()+
                                  " failed : "+e);
                     }
                  }
               }
            }
            
         }
      } else if ( evt.getActionCommand().startsWith("DIRECT:") ) {

         String directMethodName = evt.getActionCommand().substring(7);
         
         int selected = -1;
         if ( collection.getAttribute("Gui.tableView") != null ) {
            selected = table.getSelectedRow();
         } else {
            selected = wordList.getSelectedIndex();
         }            
         if (selected!=-1) {
            Object to_invoke = objects[selected];
            MethodItem directMethod =
               ClassRepository.get().getClass( to_invoke.getClass() )
               . getMethod( directMethodName );
            if( directMethod != null )
               GuiAC.invoke((Display)parent,directMethod,to_invoke,null);
         }
         
         }*/
   }

   /** Do nothing. */
   public void mouseClicked(MouseEvent me){}

   /**
    * Shows a popup if needed.
    *
    * @param e mouse event descriptor */

   public void mousePressed(MouseEvent e) {
      maybeShowPopup(e);
   }

   /**
    * Closes a popup if needed.
    *
    * @param e mouse event descriptor */

   public void mouseReleased(MouseEvent e) {
      maybeShowPopup(e);
   }

   /** Do nothing. */
   public void mouseExited(MouseEvent me){}
   /** Do nothing. */
   public void mouseEntered(MouseEvent me){}   
   /** Do nothing. */
   //public void actionPerformed(ActionEvent ae){}

   private void maybeShowPopup(MouseEvent e) {
      TreePath tp = tree.getPathForLocation( e.getX(), e.getY() );
      if ( tp != null ) {
         Object o = ((DefaultMutableTreeNode) tp.getLastPathComponent()).getUserObject();
         if (e.isPopupTrigger()) {

            tree.setSelectionPath(tp);

            if ( o instanceof Wrappee ) {
               
               JPopupMenu dynPopup = new JPopupMenu();
               JMenuItem menuItem = null;
               
               menuItem = new JMenuItem( "View" );
               menuItem.addActionListener( this );
               dynPopup.add( menuItem );
               //menuItem = new JMenuItem( "Open view (new window)..." );
               //menuItem.addActionListener( this );
               //dynPopup.add( menuItem );
                              
               ClassItem cli = ClassRepository.get().getClass(o.getClass());

               String[] menu = (String[])cli.getAttribute("Gui.attachedMenu");
               
               if( menu != null ) {
                  if(menu.length>0) dynPopup.addSeparator();
                  for(int i=0;i<menu.length;i++) {
                     if(menu[i].equals("")) dynPopup.addSeparator();
                     else {
                        menuItem = new JMenuItem(NamingConventions.textForName(menu[i]));
                        menuItem.addActionListener(this);
                        dynPopup.add(menuItem);
                     }
                  }
               } else {
                  Collection meths = cli.getMethods();
                  if(meths.size()>0) dynPopup.addSeparator();
                  Iterator it = meths.iterator();
                  while( it.hasNext() ) {                     
                     MethodItem mi = ((MethodItem[])it.next())[0];
                     // do not show the getters
                     if(mi.getAccessedFields()!=null && mi.getAccessedFields().length>0)
                        continue;
                     if( NamingConventions.isRemover(mi.getName()) ) continue;
                     if ( ClassRepository.isJacMethod( mi.getName() ) ) continue;
                     menuItem = new JMenuItem(
                        NamingConventions.textForName(mi.getName()));
                     menuItem.addActionListener( this );
                     dynPopup.add( menuItem );
                  }
               }
               dynPopup.show(e.getComponent(), e.getX(), e.getY());
            }
         }
      }
   }
 
   // interface View

   public Object getSubstance() {
      return substance;
   }

   public View getParentView() {
      return parentView;
   }

   public void refreshView() {
      System.out.println("Tree refresh");
      createTree();
   }

   public void close() {}

   public void refreshViewItem( FieldItem item, MethodItem method, Object[] args ) {
      System.out.println("Tree refresh upcalled with "+item+", "+method+", "+Arrays.asList(args));
      refreshView();
      if(NamingConventions.isAdder(method.getName())) {
         selectObject(rootNode,args[0]);
      }
   }

   public void setFocus(FieldItem field, Object extraOption) {
   }
}


class TreeNodeRenderer extends DefaultTreeCellRenderer {

   public TreeNodeRenderer() {}
   
   public Component getTreeCellRendererComponent( JTree tree,
                                                  Object value,
                                                  boolean sel,
                                                  boolean expanded,
                                                  boolean leaf,
                                                  int row,
                                                  boolean hasFocus) {
      
      super.getTreeCellRendererComponent(
         tree, value, sel, expanded, leaf, row, hasFocus);

      setDefaultAppearence( value );
      
      return this;
   }

   protected void setDefaultAppearence( Object value ) {
      DefaultMutableTreeNode node=(DefaultMutableTreeNode)value;
      Object o = node.getUserObject();
      ImageIcon icon = null;
      String text = null;
      String toolTipText = null;

      if(o==null) return;

      if( o instanceof CollectionItem ) {
         setText(((CollectionItem)o).getName());
         setIcon(null);
      } else if (o instanceof Wrappee) {
         setText(o.toString());
         ClassItem cli = ClassRepository.get().getClass(o.getClass());
         setIcon((ImageIcon) cli.getAttribute("Gui.attachedIcon"));
      }
   }

   public void actionPerformed(ActionEvent evt) {
   }

   //   class PopupListener extends MouseAdapter {
   //   }
   
}
