/*

  Copyright (C) 2001 Renaud Pawlak, Laurent Martelli
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

package jac.aspects.gui;

import java.net.URL;
import java.awt.event.*;
import java.awt.Component;
import javax.swing.*;
import jac.core.rtti.FieldItem;
import javax.swing.filechooser.*;
import java.util.Hashtable;
import java.util.Enumeration;
import java.io.File;

/**
 * This is a special value editor that allows the user to nicely edit
 * an URL. */

public class URLEditor extends JPanel implements ActionListener,ValueEditor {
   
   /** Stores the URL value. */
   protected JTextField text;

   /**
    * Constructs a new URL editor.
    */

   public URLEditor() {
      text = new JTextField(20);
      add(text);

      JButton button = new JButton ( ResourceManager.getEditIcon() );
      button.setToolTipText( "Edit" );
      button.setActionCommand( "choose" );
      button.addActionListener( this );
      add(button);
   }

   /**
    * Handles the actions performed by the users on this view.
    *
    * <p>On an URL editor, a "choose" action can be performed to allow
    * the user to open a file chooser box and to navigate within the
    * file system to choose his file.
    *
    * @param evt the performed action */

   public void actionPerformed(ActionEvent evt) {
      if (evt.getActionCommand().equals("choose")) {
         JFileChooser chooser = new JFileChooser();      
         //chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
         /*String fileSelectionMode = (String)field.getAttribute("Gui.fileSelectionMode");
         if(fileSelectionMode!=null) {
            if(fileSelectionMode.equals("DIRECTORIES_ONLY")) {
               chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
            } else if(fileSelectionMode.equals("FILES_AND_DIRECTORIES")) {
               chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
            }
         } 
         String[] fileExtensions = (String[])field.getAttribute("Gui.fileExtensions");
         String fileExtensionsDescription = 
            (String)field.getAttribute("Gui.fileExtensionsDescription");

         if(fileExtensions!=null) {
            CustomizedFileFilter filter = 
               new CustomizedFileFilter(fileExtensions,fileExtensionsDescription);
            chooser.setFileFilter(filter);
            }*/
         int returnVal = chooser.showOpenDialog( this );
         
         if ( returnVal == JFileChooser.APPROVE_OPTION ) {
            text.setText( "file:" + chooser.getSelectedFile().toString() );
         }
      }
   }

   public Object getValue() {
      try {
         return new URL(text.getText());
      } catch (Exception e) {
         return null;
      }
   }

   public void setValue(Object value) {
      if( value == null ) return;
      text.setText(((URL)value).toString());
   }

   public void setWidth(int width) {
      text.setColumns(width);
   }

   public void setHeight(int height) {}

   /**
    * Set the focus on the JTextField
    */
   public void requestFocus() {
      text.requestFocus();
      jac.util.Log.trace("gui.focus","focusing "+text.getClass().getName());
   }

   public Component getValueComponent() {
      return text;
   }

   public void onSetFocus(Object extraOption) {}

}

/**
 * A convenience implementation of FileFilter that filters out
 * all files except for those type extensions that it knows about.
 *
 * Extensions are of the type ".foo", which is typically found on
 * Windows and Unix boxes, but not on Macinthosh. Case is ignored.
 *
 * Example - create a new filter that filerts out all files
 * but gif and jpg image files:
 *
 *     JFileChooser chooser = new JFileChooser();
 *     ExampleFileFilter filter = new ExampleFileFilter(
 *                   new String{"gif", "jpg"}, "JPEG & GIF Images")
 *     chooser.addChoosableFileFilter(filter);
 *     chooser.showOpenDialog(this);
 *
 * @version 1.10 04/23/99
 * @author Jeff Dinkins
 */
class CustomizedFileFilter extends FileFilter {

    private static String TYPE_UNKNOWN = "Type Unknown";
    private static String HIDDEN_FILE = "Hidden File";

    private Hashtable filters = null;
    private String description = null;
    private String fullDescription = null;
    private boolean useExtensionsInDescription = true;

    /**
     * Creates a file filter. If no filters are added, then all
     * files are accepted.
     *
     * @see #addExtension
     */
    public CustomizedFileFilter() {
	this.filters = new Hashtable();
    }

    /**
     * Creates a file filter that accepts files with the given extension.
     * Example: new ExampleFileFilter("jpg");
     *
     * @see #addExtension
     */
    public CustomizedFileFilter(String extension) {
	this(extension,null);
    }

    /**
     * Creates a file filter that accepts the given file type.
     * Example: new ExampleFileFilter("jpg", "JPEG Image Images");
     *
     * Note that the "." before the extension is not needed. If
     * provided, it will be ignored.
     *
     * @see #addExtension
     */
    public CustomizedFileFilter(String extension, String description) {
	this();
	if(extension!=null) addExtension(extension);
 	if(description!=null) setDescription(description);
    }

    /**
     * Creates a file filter from the given string array.
     * Example: new ExampleFileFilter(String {"gif", "jpg"});
     *
     * Note that the "." before the extension is not needed adn
     * will be ignored.
     *
     * @see #addExtension
     */
    public CustomizedFileFilter(String[] filters) {
	this(filters, null);
    }

    /**
     * Creates a file filter from the given string array and description.
     * Example: new ExampleFileFilter(String {"gif", "jpg"}, "Gif and JPG Images");
     *
     * Note that the "." before the extension is not needed and will be ignored.
     *
     * @see #addExtension
     */
    public CustomizedFileFilter(String[] filters, String description) {
	this();
	for (int i = 0; i < filters.length; i++) {
	    // add filters one by one
	    addExtension(filters[i]);
	}
 	if(description!=null) setDescription(description);
    }

    /**
     * Return true if this file should be shown in the directory pane,
     * false if it shouldn't.
     *
     * Files that begin with "." are ignored.
     *
     * @see #getExtension
     * @see FileFilter#accepts
     */
    public boolean accept(File f) {
	if(f != null) {
	    if(f.isDirectory()) {
		return true;
	    }
	    String extension = getExtension(f);
	    if(extension != null && filters.get(getExtension(f)) != null) {
		return true;
	    };
	}
	return false;
    }

    /**
     * Return the extension portion of the file's name .
     *
     * @see #getExtension
     * @see FileFilter#accept
     */
     public String getExtension(File f) {
	if(f != null) {
	    String filename = f.getName();
	    int i = filename.lastIndexOf('.');
	    if(i>0 && i<filename.length()-1) {
		return filename.substring(i+1).toLowerCase();
	    };
	}
	return null;
    }

    /**
     * Adds a filetype "dot" extension to filter against.
     *
     * For example: the following code will create a filter that filters
     * out all files except those that end in ".jpg" and ".tif":
     *
     *   ExampleFileFilter filter = new ExampleFileFilter();
     *   filter.addExtension("jpg");
     *   filter.addExtension("tif");
     *
     * Note that the "." before the extension is not needed and will be ignored.
     */
    public void addExtension(String extension) {
	if(filters == null) {
	    filters = new Hashtable(5);
	}
	filters.put(extension.toLowerCase(), this);
	fullDescription = null;
    }


    /**
     * Returns the human readable description of this filter. For
     * example: "JPEG and GIF Image Files (*.jpg, *.gif)"
     *
     * @see setDescription
     * @see setExtensionListInDescription
     * @see isExtensionListInDescription
     * @see FileFilter#getDescription
     */
    public String getDescription() {
	if(fullDescription == null) {
	    if(description == null || isExtensionListInDescription()) {
 		fullDescription = description==null ? "(" : description + " (";
		// build the description from the extension list
		Enumeration extensions = filters.keys();
		if(extensions != null) {
		    fullDescription += "." + (String) extensions.nextElement();
		    while (extensions.hasMoreElements()) {
			fullDescription += ", " + (String) extensions.nextElement();
		    }
		}
		fullDescription += ")";
	    } else {
		fullDescription = description;
	    }
	}
	return fullDescription;
    }

    /**
     * Sets the human readable description of this filter. For
     * example: filter.setDescription("Gif and JPG Images");
     *
     * @see setDescription
     * @see setExtensionListInDescription
     * @see isExtensionListInDescription
     */
    public void setDescription(String description) {
	this.description = description;
	fullDescription = null;
    }

    /**
     * Determines whether the extension list (.jpg, .gif, etc) should
     * show up in the human readable description.
     *
     * Only relevent if a description was provided in the constructor
     * or using setDescription();
     *
     * @see getDescription
     * @see setDescription
     * @see isExtensionListInDescription
     */
    public void setExtensionListInDescription(boolean b) {
	useExtensionsInDescription = b;
	fullDescription = null;
    }

    /**
     * Returns whether the extension list (.jpg, .gif, etc) should
     * show up in the human readable description.
     *
     * Only relevent if a description was provided in the constructor
     * or using setDescription();
     *
     * @see getDescription
     * @see setDescription
     * @see setExtensionListInDescription
     */
    public boolean isExtensionListInDescription() {
	return useExtensionsInDescription;
    }
}
