/*

  Copyright (C) 2001 Renaud Pawlak, Laurent Martelli
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

package jac.aspects.gui;

import java.awt.Component; 

/**
 * This interface is implemented to define different value editors
 * depending on the type of the value to edit. */

public interface ValueEditor {

   /**
    * Gets the value of the edited object.
    *
    * @return an object of the edited type */

   Object getValue();

   /**
    * Sets the value of the edited object.
    *
    * @param value an instance of the edited object type */

   void setValue(Object value);

   /**
    * Sets the width of the editor.
    *
    * @param width the width in pixels or columns depending on the
    * editor type */

   void setWidth(int width);

   /**
    * Sets the height of the editor.
    *
    * @param height the height in pixels or columns depending on the
    * editor type (may have no effect) */

   void setHeight(int height);

   /**
    * Returns the component that actually holds the value.
    *
    * @return the valued component */

   Component getValueComponent();

   /**
    * Called when the focus is given to the editor
    *
    * @param extraOption an optional parameter
    */
   void onSetFocus(Object extraOption);
}

