/*

  Copyright (C) 2001 Renaud Pawlak, Laurent Martelli
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA */

package jac.aspects.gui;

import jac.core.rtti.*;

/**
 * This interface defines a generic view on JAC objects.
 *
 *
 * @author <a href="mailto:renaud@cnam.fr">Renaud Pawlak</a>
 * @author <a href="mailto:laurent@aopsys.com">Laurent Martelli</a>
 */

public interface View {

   /**
    * Returns the object shown by this view.
    *
    * @return the shown object */

   Object getSubstance();

   /**
    * Returns the parent view of this view.
    *
    * <p>The parent view is generally a view that aggregates (or
    * contains) the current view. If the current view is not contained
    * in any other view, then the parent view is null.
    *
    * @return the view that contains this view */

   View getParentView();

   /**
    * Refreshes an item of the current view.
    *
    * <p>This method is automatically called by
    * <code>ViewControlWrapper</code> when the substance state
    * that corresponds to the given field item changes.
    *
    * @param item the field item that has changed
    * @param method the method item that was called
    * @param args the arguments passed to the method
    */

   void refreshViewItem( FieldItem item, MethodItem method, Object[] args );

   /**
    * Refreshes the whole view.
    */

   void refreshView();

   /**
    * Give the focus to a specified FieldItem.
    *
    * @param field the FieldItem to give the focus to.
    */
   void setFocus(FieldItem field, Object extraOption);

   /**
    * This method is called by the system when the view is closed.
    */

   void close();
}
