/*

  Copyright (C) 2001 Laurent Martelli.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

package jac.aspects.persistence;

import java.util.*;

import jac.core.*;
import jac.util.*;

/**
 * Base class for collection wrappers
 */

public abstract class CollectionWrapper extends PersistenceWrapper {
   boolean isLoaded = false;

   public CollectionWrapper (OID oid) {
      super(oid);
   }

   /**
    * Load the whole collection if it is not already loaded
    */
   public void load() throws Exception {
      if (!isLoaded) {
         doLoad();
         isLoaded = true;
      }
   }

   /**
    * Really load the whole collection. This is an abstract method
    * must be overriden by subclasses.
    */
   protected abstract void doLoad() throws Exception;

   /**
    * Remove all instances from the collection
    */
   public Object clear() throws Exception {
      Object result = proceed();
      getStorage().clearCollection(oid);
      return result;
   }

   public Object preload() throws Exception {
      Log.trace("persistence",oid+".preload for "+method());
      load();
      return proceed();
   }
   
   public Object size() throws Exception {
      if (!isLoaded) {
         return new Integer( new Long(getStorage().getCollectionSize(oid)).intValue() );
      } else {
         return proceed();
      }
   }

   public Object isEmpty() throws Exception {
      if (!isLoaded) {
         return new Boolean(getStorage().getCollectionSize(oid)==0);
      } else {
         return proceed();
      }
   }

}
