/*

  Copyright (C) 2001 Laurent Martelli.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

/**
 * ListWrapper.java
 *
 *
 * Created: Sat Oct 13 16:14:15 2001
 *
 * @author <a href="mailto: "Laurent Martelli</a>
 * @version
 */

package jac.aspects.persistence;

import jac.core.*;
import java.util.*;
import jac.util.*;

/**
 * A wrapper for the list interface.
 */
public class ListWrapper extends CollectionWrapper {

   public ListWrapper (OID oid) {
      super(oid);
   }

   protected void doLoad() throws Exception {
      // Keep the number of dynamic invocations to the minimum
      List list = getStorage().getList(oid);
      for(int i=0; i<list.size(); i++) {
         list.set(i,normalizeOutput(list.get(i)));
      }
      wrappee().invokeOrg("addAll",new Object[] { list } );
   }

   public Object contains() throws Exception {
      if (isLoaded) {
         return proceed();
      } else {
         return new Boolean(getStorage().listContains(oid,normalizeInput(arg(0))));
      }
   }

   public Object add() throws Exception {
      if (isLoaded)
         proceed();

      if (args().length==1) {
         getStorage().addToList(oid,normalizeInput(arg(0)));
      } else if (args().length==2) {
         getStorage().addToList(oid,((Integer)arg(0)).longValue(),normalizeInput(arg(1)));
      }
      return new Boolean(true);
   }

   public Object get() throws Exception {
      if (isLoaded) {
         return proceed();
      } else {
         return normalizeOutput(getStorage().getListItem(oid,((Integer)arg(0)).longValue()));
      }
   }
   
   public Object remove() throws Exception {
      Log.trace("persistence.list", "remove : "+wrappee()+","+Arrays.asList(args()));
      Object result = null;
      if (isLoaded) {
          result = proceed();
      } else {
         // WRONG!!!
         result = new Boolean(true);
      }
      Log.trace("persistence.list", "proceeded");
      if (arg(0).getClass().equals("int")) {
         Log.trace("persistence.list", "remove position");
         getStorage().removeFromList(oid,((Integer)arg(0)).longValue());
      } else {
         Log.trace("persistence.list", "remove value");
         getStorage().removeFromList(oid,normalizeInput(arg(0)));
      }
      Log.trace("persistence.list", "value removed");
      return result;
   }
   
   public Object set() throws Exception {
      Object result = proceed();
      return result;
   }
   
   public Object indexOf() throws Exception {
      if (isLoaded)
         return proceed();
      else 
         return new Integer(
            new Long(getStorage().getIndexInList(oid,normalizeInput(arg(0)))).intValue() );
   }

   public Object lastIndexOf() throws Exception {
      if (isLoaded)
         return proceed();
      else 
         return new Integer( 
            new Long(getStorage().getLastIndexInList(oid,normalizeInput(arg(0)))).intValue() );
   }

}// ListWrapper

