/*

  Copyright (C) 2001 Laurent Martelli.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

package jac.aspects.persistence;

import java.util.*;

/**
 * A wrapper for the Map interface.
 */
public class MapWrapper extends CollectionWrapper {

   public MapWrapper(OID oid) {
      super(oid);
   }

   protected void doLoad() throws Exception {
      Map map = getStorage().getMap(oid);
      // Keep the number of dynamic invocations to the minimum
      HashMap normalizedMap = new HashMap(map.size());
      Iterator i = map.entrySet().iterator();
      while (i.hasNext()) {
         Map.Entry entry = (Map.Entry)i.next();
         normalizedMap.put(normalizeOutput(entry.getKey()),
                           normalizeOutput(entry.getValue()));
      }
      wrappee().invokeOrg("putAll",new Object[] { normalizedMap } );
   }

   public Object containsKey() throws Exception {
      if (isLoaded) {
         return proceed();
      } else {
         return new Boolean(getStorage().mapContainsKey(oid,normalizeInput(arg(0))));
      }
   }
   
   public Object containsValue() throws Exception {
      if (isLoaded) {
         return proceed();
      } else {
         return new Boolean(getStorage().mapContainsValue(oid,normalizeInput(arg(0))));
      }
   }
   
   public Object put() throws Exception {
      Object ret = null;
      if (isLoaded)
         proceed();
      ret = normalizeOutput( getStorage().putInMap(
         oid,normalizeInput(arg(0)),normalizeInput(arg(1))));
      return ret;
   }

   public Object get() throws Exception {
      if (isLoaded)
         return proceed();
      else
         return normalizeOutput(getStorage().getFromMap(oid,normalizeInput(arg(0))));
   }

   public Object remove() throws Exception {
      Object result1 = null;
      boolean proceeded = false;
      if (isLoaded) {
         result1 = proceed();
         proceeded = true;
      }
      Object result2 = getStorage().removeFromMap(oid,normalizeInput(arg(0)));
      if (proceeded)
         return result1;
      else 
         return normalizeOutput(result2);
   }
}
