/*

  Copyright (C) 2001 Laurent Martelli.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

package jac.aspects.persistence;

import jac.core.*;
import jac.util.*;

/**
 * This wrapper wraps the NameRepository 
 **/

public class NameRepositoryWrapper extends Wrapper {


   /**
    * Unregister a JacObject from the repository.
    **/

   public void unregister ( ) throws Exception {
      getStorage().deleteName((String)arg(0));
      proceed();
   }

   /**
    * Unregister a JacObject from the repository.
    *
    * @param object the object to unregister
    */


   public void unregisterObject ( ) throws Exception 
   {
      if (arg(0) instanceof Wrappee) {
         Object[] wrappers = wrappee().getWrappers( PersistenceWrapper.class );
         if (wrappers!=null && wrappers.length!=0) {
            getStorage().deleteObject(
               ((PersistenceWrapper)wrappers[0]).getOid());
         }
      }
      proceed();
   }

   /**
    * Returns true if an object is registered with this name.
    */

   public boolean isRegistered ( ) throws Exception {
      boolean res = ((Boolean)proceed()).booleanValue();
      if (!res) 
         return getStorage().getOIDFromName((String)arg(0))!=null;
      else
         return res;
   }

   /**
    * Return all the registered objects as an array.

   public Object[] getObjects () {
   }
    */


   /**
    * Return the names of the registered objects as an array.    

   public Object[] getNames () {
   }
    */

   /**
    * Return a registered object for a given logical name.
    */

   public Object getObject ( ) throws Exception {
      Object res = proceed();
      if (res==null) {
         OID oid = getStorage().getOIDFromName((String)arg(0));
         if (oid!=null) {
            return getAC().getObject(oid);
         } else {
            return null;
         }
      } else {
         return res;
      }
   }

   /**
    * Returns the name of a registered object. Null if not
    * registered.
    */

   public Object getName ( ) {
      Object res = proceed();
      Log.trace("persistence","getName on "+arg(0).getClass().getName());
      if (res==null && arg(0) instanceof Wrappee) {
         Log.trace("persistence","forceTreatment on "+arg(0));
         getAC().forceTreatment(wrappee());
         res = wrappee().invokeOrg(method(),args());
      }
      return res;
   }

   public Storage getStorage() throws Exception {
      return ((PersistenceAC)getAspectComponent()).getStorage();
   }

   public PersistenceAC getAC() {
      return (PersistenceAC)getAspectComponent();
   }

}
