/*
  Copyright (C) 2001 Laurent Martelli.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */

package jac.aspects.persistence;

import jac.core.rtti.ClassItem;

/**
 * This is the configuration interface of the persistence aspect.<p>
 *
 * @see PersistenceAC
 */

public interface PersistenceConf {   

   /**
    * Specify which storage class to use and the parameters to use to
    * instanciate it. The choice of the storage's constructor is based
    * on the number of arguments, so there should not be several
    * constructors with the same number of arguments.
    *
    * @param storageClass the storage class
    * @param storageParameters the parameters */
   
   void configureStorage( ClassItem storageClass, String[] storageParameters );
   
   /**
    * Configure persistence for one class.
    *
    * <p>The persistence type must be PersistenceAC.ROOT (a named object
    * that is an entry point in the storage) or
    * PersistenceAC.PERSISTENT for a regular persistent class.
    *
    * <p>ROOT objects are always wrapped, PERSISTENT are only wrapped
    * when they become persistent, that is when they are linked to an
    * already persistent object.
    * 
    * @param type the class to configure
    * @param persistentType the persistence type for this class */

   void configureClass( ClassItem type, String persistenceType );

   /**
    * Register a static name, that is a name that will not be changed
    * by the persistence AC.  
    *
    * @param name the static name
    */
   public void registerStatic(String name);

}
