/*

  Copyright (C) 2001 Laurent Martelli.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

package jac.aspects.persistence;

import java.sql.*;
import jac.util.*;

/**
 * Implements the storage to store within a PostgreSQL database.
 */

public class PostgresStorage extends SQLStorage {

   /**
    * Creates a new storage for a PostgreSQL database.
    *
    * @param database the database name
    * @param user the user name
    * @param password the password for this user
    */

   public PostgresStorage(String database, String user, String password) 
      throws SQLException {
      try {
         Class.forName("org.postgresql.Driver");
      } catch (ClassNotFoundException e) {
         System.out.println("*** failed to load postgresql JDBC driver : " +e);
         return;
      }
      try {
         Connection db;
         db = DriverManager.getConnection("jdbc:postgresql:"+database,user,password);
         setConnection(db);
      } catch (SQLException e) {
         Log.error("failed to connect to the database : "+e);
         e.printStackTrace();
      }
   }

   public long getNextVal(String sequence) throws Exception {
      return getLong("select nextval('"+sequence+"')");
   }

}
