/*

  Copyright (C) 2001 Laurent Martelli.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

package jac.aspects.persistence;

import java.util.*;
import jac.util.*;

/**
 * A wrapper for the Map interface.
 */
public class SetWrapper extends CollectionWrapper {

   public SetWrapper(OID oid) {
      super(oid);
   }

   protected void doLoad() throws Exception {
      // Keep the number of dynamic invocations to the minimum
      List list = getStorage().getList(oid);
      for(int i=0; i<list.size(); i++) {
         list.set(i,normalizeOutput(list.get(i)));
      }
      wrappee().invokeOrg("addAll",new Object[] { list } );
   }

   public Object contains() throws Exception {
      if (isLoaded) {
         return proceed();
      } else {
         return new Boolean(getStorage().setContains(oid,normalizeInput(arg(0))));
      }
   }

   public boolean add() throws Exception {
      if (isLoaded)
         proceed();
      return getStorage().addToSet(oid,normalizeInput(arg(0)));
   }

   public Object remove() throws Exception {
      boolean result1 = false;
      if (isLoaded) {
         result1 = ((Boolean)proceed()).booleanValue();
      }
      boolean result2 = getStorage().removeFromSet(oid,normalizeInput(arg(0)));
      if (isLoaded) {
         if (result1!=result2)
            throw new Error("SetWrapper.remove result1 != result2");
         return new Boolean(result1);
      } else {
         return new Boolean(result2);
      }
   }
}
