package jac.aspects.persistence;

import jac.util.Log;

public class ValueConverter
{

   /**
    * Returns a string representation of a field so that it can be
    * stored.<p>
    *
    * @param obj a persistent or primitive object
    * @return a ready to store string representation */

   static public String objectToString(Object obj)
   {
      if( obj == null ) return "null";
      Class lClass=obj.getClass();
      String className=lClass.getName();
      String value;
      if (obj instanceof OID) {
         value = ((OID)obj).toString();
      } else {
         if (lClass == java.lang.String.class) {
            value = (String)obj; 
         } else if (lClass == java.util.Date.class) {
            value = String.valueOf(((java.util.Date)obj).getTime());
         } else if (lClass == java.net.URL.class) {
            value = ((java.net.URL)obj).toString();
         } else {
            value = obj.toString();
         }
         value = className+":"+value;
      }
      return value;
   }

   /**
    * Returns an object from a string, depending on the needed type.<p>
    *
    * @param str the value in a string format
    * @param type the type of the string value
    * @return an object value deduced from the string representation
    * and from the needed type */

   static public Object stringToObject(String str)
   {
      Log.trace("ValueConverter","stringToObject("+str+")");
      if ( str.equals("null") ) return null;
      char firstChar = str.charAt(0);
      if (Character.isDigit(firstChar)) {
         return new LongOID(Long.parseLong(str));
      } else {
         int separator = str.indexOf(":");
         String type = str.substring(0,separator);
         str = str.substring(separator+1);
         Log.trace("storage.conv","type = "+type+" ; value = "+str);
         if (type.equals("java.lang.String")) {
            return str;
         } else if (type.equals("java.util.Date")) {
            return new java.util.Date(Long.parseLong(str));
         } else if (type.equals("boolean") || type.equals("java.lang.Boolean")) {
            if (str.equals("true"))
               return new Boolean(true);
            else if (str.equals("false"))
               return new Boolean(false);
            else {
               Log.error("BAD BOOLEAN VALUE "+str);
               return null;
            }
         } else if (type.equals("java.net.URL")) {
            try {
               return new java.net.URL(str);
            } catch (Exception e) {
               e.printStackTrace();
               return null;
            }
         } else if (type.equals("int") || type.equals("java.lang.Integer")) {
            return new Integer(str);
         } else if (type.equals("long") || type.equals("java.lang.Long")) {
            return new Long(str);
         } else if (type.equals("float") || type.equals("java.lang.Float")) {
            return new Float(str);
         } else if (type.equals("float") || type.equals("java.lang.Double")) {
            return new Double(str);
         } else {
            Log.error("Unhandled type for value `"+str+"' : "+type);
            throw new Error("Unhandled type for value `"+str+"' : "+type);
         }
      }
   }

}
