/*
  Copyright (C) 2001 Renaud Pawlak

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

package jac.aspects.session;

import jac.util.*;
import jac.core.*;
import java.lang.reflect.*;
import java.util.*;
import java.awt.*;

/**
 * This wrapper handles per-session objects within the JAC system.
 *
 * <p>For each session, it uses a different copy of the original
 * object so that each client see a different state.
 *
 * @see #handlePerSessionObject()
 * @author Renaud Pawlak */

public class PerSessionObjectWrapper extends Wrapper {

   Hashtable sessionObjects = new Hashtable(); 

   /**
    * This wrapping method handles an hashtable of copied objects.
    *
    * <p>There is one copied object per session with possibly a
    * different state from the original. The call is thus forwarded to
    * the copy that corresponds to the session.
    *
    * <p>The first time, the original object is cloned so that its
    * state is that same as the original.
    *
    * @return the result given by the per-session copy */

   public Object handlePerSessionObject() {

      Log.trace("session","handling per-session object for " + wrappee());

      String sid = (String) attr( "Session.sid" );
      
      if ( sid == null || sid.startsWith("Swing") ) {
         Log.trace("session","session is not defined by client");
         return proceed();
      } 

      Log.trace("session","found session " + sid);
      
      Object sessionObject = null;
      
      if ( sessionObjects.containsKey( sid ) ) {
         sessionObject = sessionObjects.get( sid );
      } else {
         // the initial session object has the same state as the original
         // object
         Log.trace("session","cloning object " + wrappee());
         sessionObject = wrappee().clone();
         sessionObjects.put(sid,sessionObject);
      }

      Object result = null;
      if( sessionObject == wrappee() ) {
         // we are the session object, this should not happend if
         // the session aspect is correctly defined
         Log.warning(wrappee()+
                   " is a session object and is wrapped.");
         result = proceed();
      } else {
         // we forward the call to the session object 
         // (and we do not use the original)
         Log.trace("session",wrappee()+" forwarding to session object " + 
                   sessionObject+"."+method());
         result = ((Wrappee)sessionObject).invoke(method(),args());
      }
      return result;
   }

}




