/*
  Copyright (C) 2001 Renaud Pawlak

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

package jac.aspects.session;

import jac.core.*;
import jac.wrappers.*;
import jac.aspects.naming.*;
import jac.util.*;

/**
 * This aspect component handle the session aspect within JAC
 * applications.
 *
 * <p>The session aspects memorizes some contextual informations as
 * users id or password and link them to the current session id (the
 * "Session.sid" attribute in the context). If the client correctly
 * sets this attribute for each interaction, then the session aspect
 * restores the saved information so that the user will not have to
 * input extra information (such as his password) for each
 * interaction.

 * @see jac.aspects.session.SessionWrapper
 *
 * @author Renaud Pawlak */

public class SessionAC extends AspectComponent implements SessionConf {

   public void defineSessionHandlers(String classExpr, 
                                     String methodExpr, 
                                     String objectExpr) {
      pointcut(objectExpr, classExpr, methodExpr, 
               SessionWrapper.class.getName(), "handleSession", false);
   }

   public void definePerSessionObjects(String classExpr, 
                                       String objectExpr) {
      Log.trace( "session", "defining per-session objects: "+
                 classExpr+","+objectExpr );
      pointcut(objectExpr, classExpr, "ALL", 
               PerSessionObjectWrapper.class.getName(), 
               "handlePerSessionObject", false);
   }

   String storedAttributes[] = new String[0];

   public void declareStoredAttributes(String attributes[]) {
      storedAttributes = attributes;
   }

   /**
    * Stored attributes accessor.
    *
    * @return the stored attributes
    */
   public String[] getStoredAttributes() {
      return storedAttributes;
   }

}






