/*
  Copyright (C) 2001 Renaud Pawlak

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

package jac.aspects.session;

import jac.util.*;
import jac.core.*;
import java.lang.reflect.*;
import java.util.*;
import java.awt.*;

/**
 * This wrapper handles the session for each object within the JAC system.
 *
 * @see #handleSession()
 * @author Renaud Pawlak */

public class SessionWrapper extends Wrapper {

   /** Stores the sessions and their contextual attributes (sid ->
       saved attributes). */
   protected static Hashtable sessions = new Hashtable();

   /**
    * Handles sessions for the wrapped method.
    *
    * <p>The session handling algorithm is:
    *
    * <ul>
    * <li>if the session id - <code>attr("Session.sid")</code> - is not defined, do nothing
    * <li>try to restore the saved context attributes for this session id if already saved
    * <li>else save them into the <code>sessions</code> field
    * </ul>
    *
    * @return the wrapped method return value
    */
    
   public Object handleSession() {

      Log.trace("session","handling session for " + wrappee()+"."+method());

      String sid = (String) attr( "Session.sid" );
      
      if ( sid == null ) {
         Log.trace("session","session is not defined by client");
         return proceed();
      } 

      Log.trace("session","found session " + sid);
      Hashtable savedAttributes = null;
      
      if ( sessions.containsKey( sid ) ) {
         savedAttributes = (Hashtable) sessions.get( sid );
      } else {
         savedAttributes = new Hashtable();
         sessions.put(sid,savedAttributes);
      }
      
      String[] storedAttributes = ((SessionAC)getAspectComponent()).getStoredAttributes();

      for (int i=0; i<storedAttributes.length; i++) {
         Object value = savedAttributes.get(storedAttributes[i]);
         Log.trace("session","reading "+storedAttributes[i]+" from session:" + value);
         attrdef( storedAttributes[i], value );
      }

      Object result = proceed();

      for (int i=0; i<storedAttributes.length; i++) {
         Object value = attr(storedAttributes[i]);
         if (value != null) {
            Log.trace("session","reading "+storedAttributes[i]+
                      " from session:" + value);
            savedAttributes.put( storedAttributes[i], value );
         }
      }
      
      return result;
   }
}





