/*
  Copyright (C) 2001 Renaud Pawlak

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */

package jac.aspects.synchronization;

import jac.util.Semaphore;
import jac.util.Log;
import jac.core.AspectComponent;
import jac.core.Wrapper;

/**
 * This aspect component allows the programmer to synchronize a set of
 * methods on different objects or classes in a centralized way (do
 * not use the synchronized java keyword anymore).
 *
 * <p>The monitor implementation to provide synchronization is
 * implemented by a semaphore.
 * 
 * @see jac.util.Semaphore */

public class SynchronizationAC extends AspectComponent implements SynchronizationConf {

   public void synchronize(String classes,String methods,String objects) {
      pointcut(objects,classes,methods,new SynchronizationWrapper(),"synchronize");
   }

   /**
    * This inner wrapper implements the methods synchronization with a
    * semaphore. */
   
   public class SynchronizationWrapper extends Wrapper {
      // one resource is available by default 
      // (only on thread can enter)
      Semaphore semaphore = new Semaphore(1);
      public Object synchronize() {
         Object ret = null;
         Log.trace("synchronization",wrappee()+
                   " acquires semaphore for "+method());
         semaphore.acquire();
         try {
            proceed();
         } finally {
         Log.trace("synchronization",wrappee()+
                   " releases semaphore for "+method());
            semaphore.release();
         }
         return ret;
      }
   }

}
