package jac.core;

import java.io.*;
import java.util.Vector;
import java.net.URL;
import java.net.MalformedURLException;
import jac.util.Log;

public class ParserImpl implements Parser 
{
   public Vector parse(String filePath, String targetClass) throws IOException
   {
      InputStream inputStream = null;
      try {
         Log.trace("parser","FileInputStream("+filePath+")");
         inputStream = new FileInputStream(filePath);
      } catch ( FileNotFoundException e ) {
         try {
            Log.trace("parser","getResourceAsStream("+filePath+")");
            inputStream = ClassLoader.getSystemClassLoader().getResourceAsStream(filePath);
            if (inputStream==null) {
               Log.warning("Resource "+filePath+" not found");
            }
         } catch (Exception e2 ) {
            // uncaught MalformedURLException
            Log.trace("parser","new URL("+filePath+")");
            try {
               URL url = new URL(filePath);
               inputStream = url.openStream();
            } catch (MalformedURLException e3) {
               e3.printStackTrace();
               return null;
            }
         }
      }
      if (inputStream!=null) {
         return parse(inputStream,filePath,targetClass);
      } else {
         return new Vector();
      }
   }

   public Vector parse(InputStream inputStream, String filePath, String targetClass) {
      Vector methods = null;
      if (filePath.endsWith(".xml")) {
         try {
            InputStreamParser parser = 
               (InputStreamParser)(Class.forName(
                  "jac.core.parsers.xml.JacXmlParser").newInstance());
            methods = parser.parse(inputStream, filePath, targetClass);
         } catch (Exception e) {
            Log.error("Exception during xml parsing");
            e.printStackTrace();
         }
      } else if (filePath.endsWith(".acc")) {
         try {
            InputStreamParser parser = 
               (InputStreamParser)(Class.forName(
                  "jac.core.parsers.acc.AccParserWrapper").newInstance());
            methods = parser.parse(inputStream, filePath, targetClass);
         } catch (Exception e) {
            Log.error("Exception during acc parsing");
            e.printStackTrace();
         }
      } else {
         Log.warning("No parser found for "+filePath);
         methods = new Vector();
      }
      return methods;
   }
}
