/*
  Copyright (C) 2001 Renaud Pawlak, Lionel Seinturier.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA */

package jac.core;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.*;
import jac.util.*;
import jac.core.rtti.*;


/**
 * <code>WrappeeTemplate</code> implements the set of methods that are
 * added to the JAC's wrappees when calling the
 * <code>WrappeeTranslator.translateClass()</code> method on a regular
 * class.
 *
 * <p>Most of the methods are copied into the wrappee class for each
 * method of the wrappee class. They are templated by the
 * <code>FNAME</code> identifier. This template mechanism for bytecode
 * translation allows a simple and efficient bytecode transformation.
 *
 * @see WrappeeTranslator */

public class WrappeeTemplate extends JacObject
{

   transient static protected String _FNAME_method_name;
    
   /**
    * Calls the next wrapper of the <code>FNAME</code> method. If no
    * wrapper has to be called or if all required wrappers have
    * already been called, then the original object is called (the
    * wrappee).
    *
    * @param params  the parameters of the wrapped method.
    * @param rank    the rank of the current wrapper in the wrapping chain
    * @return        the return value of the next wrapper call
    */

   public Object _nextWrapper_FNAME( Object[] params, int rank ) 
      throws Throwable 
   {
      return myNextWrapper(this, _FNAME_method_name, 
                           params, new Integer(rank));
   }

}

