/*

  Copyright (C) 2001 Renaud Pawlak

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

import jac.core.dist.*;
import java.util.Hashtable;
import java.util.Enumeration;
import java.net.InetAddress;
import java.io.*;
import java.text.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

// this is a jikes bug workaround
import jac.core.dist.RemoteContainer;
import jac.core.dist.RemoteRef;

/**
 * This class defines the servlet that handles web clients of JAC
 * applications and that can be plugged within any web server that
 * supports a servlet engine (ie Tomcat).
 *
 * <p>The default servlet has the following behavior:
 * <ul>
 * <li>binds to the local JAC server (named s0)
 * <li>resolves the WebGui server that is running on this server
 * <li>if one of these operations fails, then returns an error message
 * <li>determines the action of the request (either "view", "invoke", 
 * "new", "close", "input", or "refresh")
 * <li>gets and builds the arguments corresponding to the action
 * <li>forwards the action to the GuiWeb server
 * </ul>
 * 
 * @see doGet(HttpServletRequest,HttpServletResponse)
 *
 * @version 0.8.1
 * @author <a href="http://cedric.cnam.fr/~pawlak/index-english.html">Renaud Pawlak</a> */
 
public class JacServlet extends HttpServlet {

   public void doGet( HttpServletRequest request,
                      HttpServletResponse response )
      throws IOException, ServletException {

      HttpSession session = request.getSession( true );
      String sid = session.getId();

      response.setContentType( "text/html; charset=iso8859-1" );
      response.setHeader("Cache-Control","no-cache");
      PrintWriter out = response.getWriter();

      String jacServerName = request.getContextPath().substring(1);
      RemoteContainer jacServer = RemoteContainer.resolve(jacServerName);
      if( jacServer == null ) {
         out.println( "Error: cannot resolve JAC server : "+jacServerName ); 
         return;
      }
      RemoteRef guiWebServerRef = jacServer.bindTo ( "JAC.guiWebServer" );
      if( guiWebServerRef == null ) {
         out.println( "Error: no web-gui server running on this JAC server." );
         return;
      }

      String action = (String) request.getParameter( "action" );
      String page = null;
      if( action == null ) {
         page = noAction(request,guiWebServerRef,sid);
         if (page.length()==0)
            page = "Error: invalid protocol.";
      } else if ( action.equals( "view" ) ) {
         String jid = (String) request.getParameter( "jid" );      
         page = new String( (byte[]) guiWebServerRef.
                            invoke( "viewRequest", new Object[] { sid, jid } ) );
      }
      else if ( action.equals( "frameMode" ) ) {
         String jid1 = (String) request.getParameter( "jid1" );
         String jid2 = (String) request.getParameter( "jid2" );
         String jid3 = (String) request.getParameter( "jid3" );
         page = new String( (byte[]) guiWebServerRef.invoke( 
            "frameModeRequest", new Object[] { sid, jid1, jid2, jid3 } ) );
      }
      else if ( action.equals( "update" ) || hasActionButton(request.getParameterNames())) {
         String choice = (String) request.getParameter( "choice" );
         if( choice!=null && choice.equals("CANCEL") ) {
            // Cancel
            page = new String( (byte[]) guiWebServerRef.
                               invoke( "closeRequest", new Object[] { sid } ) );
         } else if ( choice!=null && choice.equals("OK") ) {
            // OK
            String jid = (String) request.getParameter( "jid" );      
            Map fieldsValues = getFieldValues(request,true);
            page = new String( (byte[]) guiWebServerRef.invoke( 
               "updateRequest", new Object[] { sid, jid, fieldsValues } ) );
         } else {
            // View, New, or Edit
            page = noAction(request,guiWebServerRef,sid);
         }
      } else if ( action.equals( "invoke" ) ) {
         String jid = (String) request.getParameter( "jid" );      
         String method = (String) request.getParameter( "method" );      
         page = new String( (byte[]) guiWebServerRef.invoke( 
            "invokeRequest", new Object[] { sid, jid, method } ) );
      } else if ( action.equals( "new" ) ) {
         String className = (String) request.getParameter( "class" );
         page = new String( (byte[]) guiWebServerRef.invoke( 
            "newInstanceRequest", new Object[] { sid, className } ) );
      } else if ( action.equals( "close" ) ) {
         page = new String( (byte[]) guiWebServerRef.invoke( 
            "closeRequest", new Object[] { sid } ) );
      } else if ( action.equals( "input" ) ) {
         String jid = (String) request.getParameter( "jid" );      
         String method = (String) request.getParameter( "method" );      
         String choice = (String) request.getParameter( "choice" );
         if( choice != null && choice.equals("CANCEL") ) {
            page = new String( (byte[]) guiWebServerRef.invoke( 
               "cancelInputRequest", new Object[] { sid, method } ) );
         } else {
            Vector v = new Vector();
            int i = 0;
            String cur = null;
            do {
               cur = (String) request.getParameter( "arg"+i );
               if( cur != null ) v.add( cur );
               i++;
            } while ( cur != null );
            String[] params = new String[v.size()];
            for ( i = 0; i < params.length; i++ ) {
               params[i] = (String) v.get(i);
            }
            page = new String( (byte[]) guiWebServerRef.invoke( 
               "inputRequest", new Object[] { sid, method, params } ) );
         }
      } else if ( action.equals( "refresh" ) ) {
         page = new String( (byte[]) guiWebServerRef.invoke( 
            "refreshRequest", new Object[] { sid } ) );
      } else {
         out.println( "Error: invalid action." );
         return;
      }

      out.println( page );
   }

   String noAction(HttpServletRequest request, RemoteRef guiWebServerRef, String sid) 
   {
      String page = "";
      // find the parameter with value "view" or "new"
      Enumeration params = request.getParameterNames();
      while(params.hasMoreElements()) {
         String name = (String)params.nextElement();
         if (name.startsWith("_VIEW_")) {
            name = name.substring(6);
            guiWebServerRef.invoke(
               "addContext", new Object[] { sid, getFieldValues(request,false) } );
            page += new String( (byte[]) guiWebServerRef.invoke(
               "viewRequest", new Object[] { sid, name } ) );
            break;
         } else if (name.startsWith("_VIEWREF_")) {
            name = name.substring(9);
            String jid = request.getParameter(name);
            guiWebServerRef.invoke(
               "addContext", new Object[] { sid, getFieldValues(request,false) } );
            page += new String( (byte[]) guiWebServerRef.invoke(
               "viewRequest", new Object[] { sid, jid } ) );
            break;
         } else if (name.startsWith("_NEW_")) {
            name = name.substring(5);
            guiWebServerRef.invoke(
               "addContext", new Object[] { sid, getFieldValues(request, false) } );
            page += new String( (byte[]) guiWebServerRef.invoke( 
               "newInstanceRequest", new Object[] { sid, name } ) );
            break;
         } else if (name.startsWith("_EDIT_")) {
            name = name.substring(6);
            String jid = request.getParameter("jid");
            guiWebServerRef.invoke(
               "addContext", new Object[] { sid, getFieldValues(request,false) } );
            page += new String( (byte[]) guiWebServerRef.invoke(
               "invokeRequest", new Object[] { sid, jid, name } ) );
            break;
         } else if (name.startsWith("_INVOKE_")) {
            String jid = (String) request.getParameter( "jid" );      
            String method = (String) name.substring( 8 );      
            page += new String( (byte[]) guiWebServerRef.invoke( 
               "invokeRequest", new Object[] { sid, jid, method } ) );
         }
      }
      return page;
   }

   /**
    * True if one of the parameter's name starts with _VIEW_, _NEW_, _EDIT_, _VIEWREF_ or _INVOKE_
    */
   boolean hasActionButton(Enumeration paramNames) {
      while(paramNames.hasMoreElements()) {
         String name = (String)paramNames.nextElement();
         if ( name.startsWith("_NEW_") || 
              name.startsWith("_EDIT_") || 
              name.startsWith("_INVOKE_") || 
              name.startsWith("_VIEW_") ||
              name.startsWith("_VIEWREF_") )
            return true;
      }
      return false;
   }

   /**
    * Returns a map containing parameters whose name start with _FIELD_. 
    * @param request
    * @param strip wether to strip "_FIELD_" from the keys in the returned map.
    */
   Map getFieldValues(ServletRequest request, boolean strip) {
      Enumeration names = request.getParameterNames();
      Hashtable values = new Hashtable();
      while(names.hasMoreElements()) {
         String name = (String)names.nextElement();
         if (name.startsWith("_FIELD_")) {
            values.put(strip ? name.substring(7) : name,request.getParameter(name));
         }
      }
      return values;
   }
}




