/*
  JAC-Core version 0.8.1

  Renaud Pawlak, pawlak@cnam.fr, CEDRIC Laboratory, Paris, France.
  Lionel Seinturier, Lionel.Seinturier@lip6.fr, LIP6, Paris, France.

  JAC-Core is free software. You can redistribute it and/or modify it
  under the terms of the GNU Library General Public License as
  published by the Free Software Foundation.
  
  JAC-Core is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

  This work uses the Javassist system - Copyright (c) 1999-2000
  Shigeru Chiba, University of Tsukuba, Japan.  All Rights Reserved.  */


package jac.core.rtti;

import jac.core.AspectComponent;
import jac.core.Wrappee;
import jac.core.JacObject;
import jac.util.Log;

import java.util.Vector;

/**
 * This class defines the rtti aspect.
 *
 * <p>It allows the programmer to add some runtime type informations on
 * the classes of its applications.
 *
 * @see ClassItem
 * @see MethodItem
 * @see FieldItem
 * @see CollectionItem
 *
 * @author Renaud Pawlak
 * @author Laurent Martelli
 * @version 0.8.1 */

public class RttiAC extends AspectComponent implements RttiConf {

   public void addWrittenFields( String className, String methodName, 
                                 String[] writtenFields ) {
      MethodItem method = ClassRepository.get().getClass(className)
         .getMethod(methodName);
      for( int i=0; i<writtenFields.length; i++ ) {
         FieldItem fi = ClassRepository.get().getClass(className)
            .getField(writtenFields[i]);
         method.addWrittenField( fi );
      }

   }

   public void addAccessedFields( String className, String methodName, 
                                  String[] accessedFields ) {
      MethodItem method = ClassRepository.get().getClass(className)
         .getMethod(methodName);
      for( int i=0; i<accessedFields.length; i++ ) {
         FieldItem fi = ClassRepository.get().getClass(className)
            .getField(accessedFields[i]);
         method.addAccessedField( fi );
      }

   }

   public void setFieldType( String className, String fieldName, String type) 
   {
      FieldItem field = ClassRepository.get().getClass(className)
         .getField(fieldName);
      field.setAttribute("rtti.fieldType",ClassRepository.get().getObject(type));
      Log.trace("rttiac","setFieldType("+className+")");
   }

   public static MetaItem getFieldType(FieldItem field) {
      Object tmp = field.getAttribute("rtti.fieldType");
      if ( tmp != null)
         return (MetaItem)tmp;
      else 
         return null;
   }

   public void setParametersType( String className, String methodName, 
                                  String[] types) 
   {
      ClassRepository cr = ClassRepository.get();
      MethodItem method = cr.getClass(className)
         .getMethod(methodName);
      MetaItem[] metaItems = new MetaItem[types.length];
      for (int i=0; i<types.length;i++) {
         metaItems[i] = (MetaItem)cr.getObject(types[i]);
      }
      method.setAttribute("rtti.parametersType",metaItems);
   }

   public void newVirtualClass( String className )
   {
      Log.trace("rttiac","newVirtualClass("+className+")");
      ClassRepository.get().register(className,
                                     new VirtualClassItem(className));
   }

   public void declareAssociation( ClassItem classA, String fieldNameA, 
                                   ClassItem classB, String fieldNameB )
   {
      Log.trace("associations","declareAssociation "+
                classA.getName()+"."+fieldNameA+" <-> "+
                classB.getName()+"."+fieldNameB);
      FieldItem fieldA = classA.getField(fieldNameA);
      FieldItem fieldB = classB.getField(fieldNameB);
      fieldA.setAttribute("rtti.oppositeRelation",fieldB);
      fieldB.setAttribute("rtti.oppositeRelation",fieldA);
      
   }

   public void defineRepository( ClassItem type, 
                                 String repositoryName,
                                 CollectionItem repositoryCollection ) 
   {
      type.setAttribute("Rtti.repositoryName", repositoryName);
      type.setAttribute("Rtti.repositoryCollection", repositoryCollection);
   }

   static Vector roots = new Vector();

   public void addRoot(String name) {
      roots.add(name);
   }

   /**
    * Returns all the root objects defined by the aspect configuration.
    *
    * @return a vector containing the roots names
    * @see #addRoot(String) */

   public static Vector getRoots() {
      return roots;
   }

   public void setClonedFields(String className, String[] fields) {
      ClassRepository.get().getClass(className)
         .setAttribute("rtti.clonedFields",fields);
   }

   public void whenClone ( Wrappee cloned, Wrappee clone ) {
      
      ClassItem cli = ClassRepository.get().getClass(cloned.getClass());
      String[] clonedFields = (String[])cli.getAttribute("rtti.clonedFields");
      
      if( clonedFields != null ) {
         for( int i=0; i<clonedFields.length; i++ ) {
            FieldItem fi = cli.getField( clonedFields[i] );
            Log.trace("rtti","cloning field "+clonedFields[i]);
            try {
               fi.set(clone,((JacObject)fi.get(cloned)).clone());
            } catch( Exception e ) {}
         }
      }
   }

}

