
package jac.ide;

import java.io.File;
import java.io.FileWriter;
import java.io.Writer;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import jac.lib.java.util.Vector;
import java.util.Iterator;
import java.util.Map;
import java.util.List;
import java.util.Hashtable;
import jac.core.rtti.NamingConventions;
import jac.core.Display;
import jac.core.JacObject;
import jac.util.LineNumberWriter;
import jac.util.Log;

import gnu.regexp.RE;
import gnu.regexp.REMatch;

public class CodeGeneration {

   public static void createProjectCode(Project p,String path) {
      Iterator it = p.getPackages().iterator();
      while(it.hasNext()) {
         Package cur = (Package)it.next();
         createPackageCode(p,cur,path+"/"+cur.getName(),cur.getName());
      }
      it = p.getApplications().iterator();
      while(it.hasNext()) {
         Application cur = (Application)it.next();
         createApplicationCode(p,cur,path+"/"+cur.getName());
      }
   }

   public static void createApplicationCode(Project prj, 
                                            Application app,
                                            String path) {
      try {
         File f = new File(path+"/Run.java");
         if ( f.exists() )
            f.delete();
         f.getParentFile().mkdirs();
         f.createNewFile();
         FileWriter fw = new FileWriter( f );
         
         fw.write( "\npackage "+app.getName()+";\n" );
         Iterator it = app.getExternalPaths().iterator();
         while(it.hasNext()) {
            String cur = (String)it.next();
            fw.write("\nimport "+cur+";");
         }
         fw.write("\nimport jac.core.*;");
         fw.write("\n");
         fw.write( "\npublic class Run {");
         fw.write( "\n    public static void main(String[] args) {" );
         fw.write( "\n    Application app = ApplicationRepository.get().getApplication(\""+app.getName()+"\");" );
         fw.write( "\n    ACManager acm = (ACManager)ACManager.get();" );

         it = app.getAspectConfigurations().iterator();
         while(it.hasNext()) {
            AspectConfiguration cur = (AspectConfiguration)it.next();
            if( !((jac.core.ACManager)jac.core.ACManager.get()).isACDeclared(cur.getName()) ) {
               fw.write( "\n    acm.declareAC(\""+cur.getName()+"\",\""+
                         cur.getAspect().getContainer().getPPath()+"."+
                         cur.getAspect().getName()+"AC\");");
            }  
            fw.write( "\n    app.addAcConfiguration(new ACConfiguration(app,\""+
                      cur.getName()+"\",\""+path+"/"+cur.getName()+".acc\","+
                      cur.isWoven()+"));");
         }
         fw.write( "\n    app.init();");
         fw.write( "\n"+app.getLaunchingCode() );
         fw.write( "\n    }");
         fw.write( "\n}");
         fw.close();
      } catch(Exception e) {
         e.printStackTrace();
      }
      Iterator it = app.getAspectConfigurations().iterator();
      while( it.hasNext() ) {
         AspectConfiguration ac = (AspectConfiguration) it.next();
         createACFile(ac,path);
         createDefaultACFile(prj,ac,path);
      }
      createJACFile(prj,app,path);
   }

   static void writeWarning(Writer fw) {
      try {
         fw.write( "//\n" +
                   "// WARNING: this file has been automatically generated by JAC\n" +
                   "// DO NOT MODIFY unless you know what you are doing\n" +
                   "//\n" );
      } catch(Exception e) {
         e.printStackTrace();
      }
   }
      
   static void createACFile(AspectConfiguration ac, String path) {
      try {
         File f = new File(path+"/"+ac.getName()+".acc");
         if ( f.exists() )
            f.delete();
         f.getParentFile().mkdirs();
         f.createNewFile();
         FileWriter fw = new FileWriter( f );
         
         if( ac.getDefaultConfigurationCode()!=null && 
             !ac.getDefaultConfigurationCode().equals("")) {
            fw.write( "include "+path+"/"+ac.getName()+"_default.acc\n" );
         }
         fw.write( ac.getConfigurationCode() );
         fw.close();
      } catch(Exception e) {
         e.printStackTrace();
      }
   }

   static void createDefaultACFile(Project prj,AspectConfiguration ac, String path) {
      try {
         File f = new File(path+"/"+ac.getName()+"_default.acc");
         if ( f.exists() )
            f.delete();
         f.getParentFile().mkdirs();
         f.createNewFile();
         FileWriter fw = new FileWriter( f );
         
         if( ac.getDefaultConfigurationCode()!=null && 
             !ac.getDefaultConfigurationCode().equals("")) {
            Vector classes=new Vector();
            createClassesList(prj,"",classes);
            Iterator itClasses=classes.iterator();
            while(itClasses.hasNext()) {
               String classname=(String)itClasses.next();
               fw.write("class "+classname+" {\n");
               fw.write(ac.getDefaultConfigurationCode());
               fw.write("\n}\n");
            }
         }
         fw.close();
      } catch(Exception e) {
         e.printStackTrace();
      }
   }

   static void createClassesList(Project p,String path,Vector list) {
      Iterator it = p.getPackages().iterator();
      while(it.hasNext()) {
         Package cur = (Package)it.next();
         createClassesList(cur,cur.getName(),list);
      }
   }

   static void createClassesList(Package p,String path,Vector list) {
      Iterator it = p.getSubPackages().iterator();
      while(it.hasNext()) {
         Package cur = (Package)it.next();
         createClassesList(cur,path+"."+cur.getName(),list);
      }
      it = p.getClasses().iterator();
      while(it.hasNext()) {
         Class cur = (Class)it.next();
         list.add(path+"."+cur.getName());
      }
   }

   static void createJACFile(Project prj, Application app, String path) {
      try {
         File f = new File(path+"/"+app.getName()+".jac");
         if ( f.exists() )
            f.delete();
         f.getParentFile().mkdirs();
         f.createNewFile();
         FileWriter fw = new FileWriter( f );
         
         fw.write( "applicationName: "+app.getName()+"\n"+
                   "launchingClass: "+app.getName()+".Run\n");
         fw.close();
      } catch(Exception e) {
         e.printStackTrace();
      }
   }

   public static void createPackageCode(Project prj, 
                                        Package p,
                                        String path,String ppath) {
      Iterator it = p.getSubPackages().iterator();
      while(it.hasNext()) {
         Package cur = (Package)it.next();
         createPackageCode(prj,cur,path+"/"+cur.getName(),ppath+"."+cur.getName());
      }
      it = p.getClasses().iterator();
      while(it.hasNext()) {
         Class cur = (Class)it.next();
         createClassCode(prj,cur,path,ppath);
      }
      it = p.getAspects().iterator();
      while(it.hasNext()) {
         Aspect cur = (Aspect)it.next();
         createAspectCode(prj,cur,path,ppath);
      }
   }

   static Hashtable classesLineNumbers = new Hashtable();

   public static Map getClassLineNumbers(Class c) {
      Map map = (Map)classesLineNumbers.get(c);
      if (map==null) {
         map =new Hashtable();
         classesLineNumbers.put(c,map);
      }
      return map;
   }

   public static void createClassCode(Project prj,Class c,
                                      String path,String ppath) {
      try {
         Map lineNumbers = getClassLineNumbers(c);
         lineNumbers.clear();
         print("generating "+c.getName());
         File f = new File( 
            path+"/"+c.getName()+".java" );
         classFiles.put(f.getAbsolutePath(),c);
         if ( f.exists() )
            f.delete();
         f.getParentFile().mkdirs();
         f.createNewFile();
         String shortName = c.getName(); //c.getName().substring( c.getName().lastIndexOf('.') + 1 );
         LineNumberWriter fw = new LineNumberWriter(new FileWriter( f ));
         writeWarning(fw);
         fw.write( "\npackage "+ppath+";\n" );
         fw.write("\n/**\n"+
                  c.getDescription()+
                  "\n*/\n");
         fw.write( "\npublic class "+shortName+((c.getSuperClass() == null)? "" : " extends " +
                                                c.getSuperClass() ));
 
         //                   ((c.getInterfaces().size() == 0)? "" : " implements " +
         //           vectorToString( c.getInterfaces()  )) );
         fw.write( " {\n" );
         Vector ms = c.getMethods();
         for ( int i = 0; i < ms.size(); i++ ) {
            Method method=(Method)ms.get(i);
            fw.write("\n/**\n"+
                     method.getDescription());
            Iterator params=method.getParameters().iterator();
            while(params.hasNext()) {
               Parameter param=(Parameter)params.next();
               fw.write("\n@param "+param.getDescription());
            }
            fw.write("\n*/\n");
            lineNumbers.put(method,new Integer(fw.getLines()+1));
            fw.write( "\n    public " + method.getPrototype() + " {" );
            if(method.getBody()!=null)
               fw.write( "\n    " + method.getBody() );
            fw.write( "\n    }\n" );
         }

         Vector fs = c.getFields();
         for ( int i = 0; i < fs.size(); i++ ) {
            Field field=(Field)fs.get(i);
            fw.write( "\n    " + field.getPrototype()+";\n" );
            fw.write("\n/**\n"+
                     "Sets the value of field "+field.getName()+".\n"+
                     "@param "+field.getName()+" "+field.getDescription()+
                     "\n*/\n");
            createSetter(fw,field.getName(),field.getType().getFullName());
            fw.write("\n/**\n"+
                     "Gets the value of field "+field.getName()+".\n"+
                     "@return "+field.getDescription()+
                     "\n*/\n");
            createGetter(fw,field.getName(),field.getType().getFullName());
         }

         Vector rs = c.getRelationLinks();
         for ( int i = 0; i < rs.size(); i++ ) {
            RelationLink rel=(RelationLink)rs.get(i);
            fw.write( "\n    " + rel.getEndPrototype()+";\n" );
            createSetter(fw,rel.getEndRole(),
                         rel.getEndType().getFullName());
            createGetter(fw,rel.getEndRole(),
                         rel.getEndType().getFullName());

            if(rel.isEndMultiple()) {
               createAdder(fw,rel.getEndRole(),
                           rel.getEnd().getFullName());
               createRemover(fw,rel.getEndRole(),
                             rel.getEnd().getFullName());
            }
         }         

         rs = c.getEndingLinks();
         for ( int i = 0; i < rs.size(); i++ ) {
            if(!(rs.get(i) instanceof RelationLink)) continue;
            RelationLink rel=(RelationLink)rs.get(i);
            fw.write( "\n    " + rel.getStartPrototype()+";\n" );
               createSetter(fw,rel.getStartRole(),
                            rel.getStartType().getFullName());
               createGetter(fw,rel.getStartRole(),
                            rel.getStartType().getFullName());

            if(rel.isStartMultiple()) {
               createAdder(fw,rel.getStartRole(),
                           rel.getStart().getFullName());
               createRemover(fw,rel.getStartRole(),
                             rel.getStart().getFullName());
            }
         }         

         fw.write( "}\n" );
         fw.close();
         print("generation fullfilled.");
      } catch( Exception e) {
         e.printStackTrace();
      }
   }


   public static void createAspectCode(Project prj,Aspect a,
                                       String path,String ppath) {
      try {
         print("generating "+a.getName());
         File f = new File( 
            path+"/"+a.getName()+"AC.java" );
         if ( f.exists() )
            f.delete();
         f.getParentFile().mkdirs();
         f.createNewFile();
         String shortName = a.getName(); //c.getName().substring( c.getName().lastIndexOf('.') + 1 );
         FileWriter fw = new FileWriter( f );
         writeWarning(fw);
         fw.write( "\npackage "+ppath+";\n" );
         fw.write( "import jac.core.*;\n" );
         fw.write( "import jac.core.rtti.*;\n" );
         fw.write( "import jac.util.*;\n" );
         fw.write( "\npublic class "+shortName+"AC extends jac.core.AspectComponent");
         fw.write( " {\n" );
         
         fw.write( "\n    public "+shortName+"AC() {");
         Iterator it = a.getPointcutLinks().iterator();
         while(it.hasNext()) {
            PointcutLink pc = (PointcutLink)it.next();
            if(pc.getEnd() instanceof jac.ide.Class) {
               fw.write("\n        pointcut(\""+
                        "ALL\",\""+
                        ((jac.ide.Class)pc.getEnd()).getFullName()+"\",\""+
                        pc.getMethodPCD()+"\","+
                        "new "+shortName+"Wrapper()"+",\""+
                        pc.getAspectRole()+"\",\""+
                        pc.getHostPCD()+"\");");
            } else if (pc.getEnd() instanceof Instance) {
               fw.write("\n        pointcut(\""+
                        ((Instance)pc.getEnd()).getName()+"\",\""+
                        ((jac.ide.Class)((Instance)pc.getEnd()).getType()).getFullName()+"\",\""+
                        pc.getMethodPCD()+"\","+
                        "new "+shortName+"Wrapper()"+",\""+
                        pc.getAspectRole()+"\",\""+
                        pc.getHostPCD()+"\");");
            }
         }
         fw.write("\n    }");

         fw.write( "\n    public class "+shortName+"Wrapper extends jac.core.Wrapper {");

         Vector ms = a.getMethods();
         for ( int i = 0; i < ms.size(); i++ ) {
            Method method=(Method)ms.get(i);
            fw.write( "\n    public " + method.getPrototype() + " {" );
            fw.write( "\n    " + method.getBody() );
            fw.write( "\n    }\n" );
         }

         Vector fs = a.getFields();
         for ( int i = 0; i < fs.size(); i++ ) {
            Field field=(Field)fs.get(i);
            fw.write( "\n    " + field.getPrototype()+";\n" );
            //createSetter(fw,field);
            //createGetter(fw,field);
         }

         /*Vector rs = a.getRelationLinks();
         for ( int i = 0; i < rs.size(); i++ ) {
            RelationLink rel=(RelationLink)rs.get(i);
            fw.write( "\n    " + rel.getPrototype()+";\n" );
            createSetter(fw,rel);
            createGetter(fw,rel);
            
            if(rel.isMultiple()) {
               createAdder(fw,rel);
               createRemover(fw,rel);
            }
            } */        

         fw.write("\n    }");
         fw.write( "}\n" );
         fw.close();
         print("generation fullfilled.");
      } catch( Exception e) {
         e.printStackTrace();
      }
   }


   static void createGetter(Writer fw, String name, String type) {
      try {
         fw.write("\n    public "+type+" get"+
                  NamingConventions.capitalize(name)+"() {\n");
         fw.write("        return "+name+";\n");
         fw.write("    }\n");
      } catch( Exception e) {
         e.printStackTrace();
      }
   }

   static void createSetter(Writer fw, String name, String type) {
      try {
         fw.write("\n    public void set"+
                  NamingConventions.capitalize(name)+"("+
                  type+" value) {\n");
         fw.write("        this."+name+"=value;\n");
         fw.write("    }\n");
      } catch( Exception e) {
         e.printStackTrace();
      }
   }

   static void createAdder(Writer fw, String name, String type) {
      try {
         String fn = NamingConventions.getNormalizedString( 
            name.substring(0, name.length() - 1) );
         if( fn.endsWith("ie") ) {
            fn = fn.substring(0, fn.length()-2)+"y";
         }
         fw.write("\n    public void add"+
                  fn+"("+type+" value) {\n");
         fw.write("        "+name+".add(value);\n");
         fw.write("    }\n");
      } catch( Exception e) {
         e.printStackTrace();
      }
   }

   static void createRemover(Writer fw, String name, String type) {
      try {
         String fn = NamingConventions.getNormalizedString( 
            name.substring(0, name.length() - 1) );
         if( fn.endsWith("ie") ) {
            fn = fn.substring(0, fn.length()-2)+"y";
         }
         fw.write("\n    public void remove"+
                  fn+"("+type+" value) {\n");
         fw.write("        "+name+".remove(value);\n");
         fw.write("    }\n");
      } catch( Exception e) {
         e.printStackTrace();
      }
   }

   private static String vectorToString( Vector vector ) {
      if ( vector.size()==0 ) return "";
      String ls = vector.toString();
      return ls.substring( 1, ls.length() - 1 );
   }

   static void createJavaFilesList(Project p,String path,StringBuffer list) {
      Iterator it = p.getPackages().iterator();
      while(it.hasNext()) {
         Package cur = (Package)it.next();
         createJavaFilesList(cur,path+"/"+cur.getName(),list);
      }
   }

   static void createJavaFilesList(Package p,String path,StringBuffer list) {
      Iterator it = p.getSubPackages().iterator();
      while(it.hasNext()) {
         Package cur = (Package)it.next();
         createJavaFilesList(cur,path+"/"+cur.getName(),list);
      }
      it = p.getClasses().iterator();
      while(it.hasNext()) {
         Class cur = (Class)it.next();
         list = list.append(path+"/"+cur.getName()+".java ");
      }
      it = p.getAspects().iterator();
      while(it.hasNext()) {
         Aspect cur = (Aspect)it.next();
         list = list.append(path+"/"+cur.getName()+"AC.java ");
      }
   }

   static void createPackageFilesList(Project p,String path,StringBuffer list) {
      Iterator it = p.getPackages().iterator();
      while(it.hasNext()) {
         Package cur = (Package)it.next();
         list=list.append(path+"/"+cur.getName()+" ");
      }
   }

   /* filename -> Class */
   static Hashtable classFiles = new Hashtable();
   /**
    * Get the class corresponding to a filename
    */
   static Class filenameToClass(String filename) {
      Class cl = null;
      if ((cl = (Class)classFiles.get(filename))!=null) {
         return cl;
      } else {
         Log.error("Cannot find Class for filename "+filename);
         return null;
      }
   }

   /**
    * Returns a list of Errors produced by the compilation process
    */
   public static Errors compileProjectCode(Project p,String path) {
      StringBuffer list = new StringBuffer("");
      createJavaFilesList(p,path,list);
      File f = new File( path+"/classes" );
      if ( !f.exists() )
         f.mkdir();
      String cmd = "javac -d "+path+"/classes "+list;
      System.out.println("CMD = "+cmd);
      print("compiling project "+p+"...");
      Vector errors = new Vector();
      try {
         Process comp = Runtime.getRuntime().exec(cmd);
         comp.waitFor();
         BufferedReader reader = new BufferedReader(new InputStreamReader(comp.getErrorStream()));
         String line;
         RE re = new RE("^([^:]+):([0-9]+)");
         while((line=reader.readLine())!=null) {
            System.out.println(line);
            REMatch match;
            if ((match=re.getMatch(line))!=null) {
               System.out.println("New ERROR : "+match.toString(1)+":"+match.toString(2));
               errors.add(new Error(match.toString(1),
                                    Integer.parseInt(match.toString(2)),
                                    line,
                                    filenameToClass(match.toString(1))));
            }
         }
         System.out.println("Errors : "+errors);
      } catch(Exception e) {
         e.printStackTrace();
      }
      Iterator it = p.getApplications().iterator();
      while( it.hasNext() ) {
         Application app = (Application)it.next();
         print("compiling application launcher for "+app.getName()+"...");

         /*String classPath = "";
         Iterator it2 = app.getExternalPaths().iterator();
         while(it2.hasNext()) {
            String cur = (String)it2.next();
            classPath = classPath+cur;
            if( it2.hasNext() ) {
               classPath = classPath+":";
            }
            }*/
         cmd = "javac -d "+path+"/classes -classpath "+path+"/classes:"+
            jac.core.Jac.getJacRoot()+"classes "+
            path+"/"+app.getName()+"/Run.java";
         System.out.println("CMD = "+cmd);
         try {
            Process comp = Runtime.getRuntime().exec(cmd);
            comp.waitFor();
            byte[] res;
            comp.getErrorStream().read(
               res = new byte[comp.getErrorStream().available()],0,
               comp.getErrorStream().available());
            System.out.write(res);
         } catch(Exception e) {
            e.printStackTrace();
         }
      }
      print("compilation done.");
      return new Errors(errors);
   }


   public static void documentProjectCode(Project p,String path) {
      StringBuffer list = new StringBuffer("");
      createPackageFilesList(p,path,list);
      File f = new File( path+"/doc" );
      if ( !f.exists() )
         f.mkdir();
      String cmd = "javadoc -d "+path+"/classes "+list;
      System.out.println("CMD = "+cmd);
      // to do...
   }


   static void print(String message) {
      Display display = (Display)JacObject.getCollaboration()
         .getAttribute("Gui.display");
      display.showStatus(message);
   }

}
