
package jac.ide.diagrams;

import javax.swing.*;
import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import java.net.*;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.standard.*;
import CH.ifa.draw.figures.*;
import CH.ifa.draw.util.*;
import jac.util.Log;
import jac.core.Jac;
import jac.aspects.gui.CustomizedGUI;
import jac.aspects.gui.View;

public class DiagramApplet
   extends JApplet
   implements DrawingEditor, PaletteListener, 
   VersionRequester, View {
   

   static private final String JACIMAGES = "/jac/aspects/gui/resources/hotdraw/";
   
   private transient Drawing         fDrawing;
   private transient Tool            fTool;
   
   private transient DrawingView     fView;
   private transient ToolButton      fDefaultToolButton;
   private transient ToolButton      fSelectedToolButton;
   
   private transient boolean         fSimpleUpdate;
   private transient JButton          fUpdateButton;
   
   private transient JComboBox          fFrameColor;
   private transient JComboBox          fFillColor;
   private transient JComboBox          fTextColor;
   private transient JComboBox          fArrowChoice;
   private transient JComboBox          fFontChoice;
   
   private transient Thread          fSleeper;
   private Iconkit                   fIconkit;
   private transient 			UndoManager myUndoManager;
   
   static String                     fgUntitled = "untitled";
   
   private static final String       fgDrawPath = "/CH/ifa/draw/";
   //	public static final String        IMAGES = fgDrawPath+"images/";
   public static final String        IMAGES = "/jac/aspects/gui/resources/hotdraw/";
   private StorageFormatManager	fStorageFormatManager;
   
   JPanel classPalette = null;
   JPanel aspectPalette = null;
   JPanel instancePalette = null;
   JPanel groupPalette = null;
   JPanel currentPalette = null;

   ToolButton classDefaultTool = null;
   ToolButton aspectDefaultTool = null;
   ToolButton instanceDefaultTool = null;
   ToolButton groupDefaultTool = null;

   jac.ide.Diagram substance;

   // View interface
   public View getParentView() {
      return null;
   }

   public void refreshViewItem(jac.core.rtti.FieldItem field
                               , jac.core.rtti.MethodItem method, Object[] args) {
   }

   public void refreshView() {
   }

   public void close() {
      System.out.println("CLOSING DIAGRAM "+this);
      save();
      DiagramView.diagramViews.remove(this);
      stop();
   }
   
   public void setFocus(jac.core.rtti.FieldItem field, Object extraOption) {}

   /**
    * Get the value of substance.
    * @return value of substance.
    */
   public Object getSubstance() {
      return substance;
   }
   
   /**
    * Set the value of substance.
    * @param v  Value to assign to substance.
    */
   public void setSubstance(jac.ide.Diagram  v) {
      this.substance = v;
   }

   public ClassFigure findClass(jac.ide.Class _class) {
      FigureEnumeration figs = view().drawing().figures();
      while(figs.hasMoreElements()) {
         Figure fig = figs.nextFigure();
         if( (fig instanceof ClassFigure) && 
             ((ClassFigure)fig).getSubstance() == _class) {
            return (ClassFigure)fig;
         }
      }
      return null;
   }

   public Figure findElement(jac.ide.TypedElement te) {
      FigureEnumeration figs = view().drawing().figures();
      while(figs.hasMoreElements()) {
         Figure fig = figs.nextFigure();
         if( (fig instanceof View) && 
             ((View)fig).getSubstance() == te) {
            return (Figure)fig;
         }
      }
      return null;
   }

   public Figure findElement(jac.ide.ModelElement element) {
      FigureEnumeration figs = view().drawing().figures();
      while(figs.hasMoreElements()) {
         Figure fig = figs.nextFigure();
         if( (fig instanceof View) && 
             ((View)fig).getSubstance() == element) {
            return fig;
         }
      }
      return null;
   }
   
	/**
	 * Initializes the applet and creates its contents.
	 */
	public void init() {
		getVersionControlStrategy().assertCompatibleVersion();
		setUndoManager(new UndoManager());
		
		fIconkit = new Iconkit(this);

		getContentPane().setLayout(new BorderLayout());

		fView = createDrawingView();
		
		//JPanel attributes = createAttributesPanel();
		//createAttributeChoices(attributes);
		//getContentPane().add("North", attributes);

                buildClassBar();

		getContentPane().add("Center", 
                                     new JScrollPane(((Component)view())));
		JPanel buttonPalette = createButtonPanel();
		createButtons(buttonPalette);
		getContentPane().add("North", buttonPalette);

		fStorageFormatManager = createStorageFormatManager();

		initDrawing();
		// JFC should have its own internal double buffering...
		//setBufferedDisplayUpdate();
		//setupAttributes();
                load();
                //((JComponent)drawing()).setMinimumSize(new Dimension(2000,2000));
	}

   public DiagramApplet(Object substance) {
      super();
      this.substance = (jac.ide.Diagram)substance;
   }

	public void addViewChangeListener(ViewChangeListener vsl) {
	}

	public void removeViewChangeListener(ViewChangeListener vsl) {
	}

	/**
	 * Return the StorageFormatManager for this application.The StorageFormatManager is
	 * used when storing and restoring Drawing from the file system.
	 */
	public StorageFormatManager getStorageFormatManager() {
		return fStorageFormatManager;
	}

	public StorageFormatManager createStorageFormatManager() {
		StorageFormatManager storageFormatManager = new StorageFormatManager();
		storageFormatManager.setDefaultStorageFormat(new StandardStorageFormat());
		storageFormatManager.addStorageFormat(storageFormatManager.getDefaultStorageFormat());
		storageFormatManager.addStorageFormat(new SerializationStorageFormat());
		return storageFormatManager;
	}

	/*
	 * Gets the iconkit to be used in the applet.
	 */

	 /**** not sure whether this will still be needed on 1.1 enabled browsers
	 protected Iconkit iconkit() {
		if (fIconkit == null) {

			startSleeper();
			loadAllImages(this); // blocks until images loaded
			stopSleeper();
		}
		return fIconkit;
	} */

	/**
	 * Creates the attributes panel.
	 */
	protected JPanel createAttributesPanel() {
		JPanel panel = new JPanel();
		panel.setLayout(new PaletteLayout(2, new Point(2,2), false));
		return panel;
	}

	/**
	 * Creates the attribute choices. Override to add additional
	 * choices.
	 */
	protected void createAttributeChoices(JPanel panel) {
		panel.add(new JLabel("Fill"));
		fFillColor = createColorChoice("FillColor");
		panel.add(fFillColor);

		panel.add(new JLabel("Text"));
		fTextColor = createColorChoice("TextColor");
		panel.add(fTextColor);

		panel.add(new JLabel("Pen"));
		fFrameColor = createColorChoice("FrameColor");
		panel.add(fFrameColor);

		panel.add(new JLabel("Arrow"));
		CommandChoice choice = new CommandChoice();
		fArrowChoice = choice;
		choice.addItem(new ChangeAttributeCommand("none",     "ArrowMode", new Integer(PolyLineFigure.ARROW_TIP_NONE),  this));
		choice.addItem(new ChangeAttributeCommand("at Start", "ArrowMode", new Integer(PolyLineFigure.ARROW_TIP_START), this));
		choice.addItem(new ChangeAttributeCommand("at End",   "ArrowMode", new Integer(PolyLineFigure.ARROW_TIP_END),   this));
		choice.addItem(new ChangeAttributeCommand("at Both",  "ArrowMode", new Integer(PolyLineFigure.ARROW_TIP_BOTH),  this));
		panel.add(fArrowChoice);

		panel.add(new JLabel("Font"));
		fFontChoice = createFontChoice();
		panel.add(fFontChoice);
	}

	/**
	 * Creates the color choice for the given attribute.
	 */
	protected JComboBox createColorChoice(String attribute) {
		CommandChoice choice = new CommandChoice();
		for (int i = 0; i < ColorMap.size(); i++)
			choice.addItem(
				new ChangeAttributeCommand(
					ColorMap.name(i),
					attribute,
					ColorMap.color(i),
					this
				)
			);
		return choice;
	}

	/**
	 * Creates the font choice. The choice is filled with
	 * all the fonts supported by the toolkit.
	 */
	protected JComboBox createFontChoice() {
		CommandChoice choice = new CommandChoice();
		String fonts[] = Toolkit.getDefaultToolkit().getFontList();
		for (int i = 0; i < fonts.length; i++) {
			choice.addItem(new ChangeAttributeCommand(fonts[i], "FontName", fonts[i],  this));
		}
		return choice;
	}

	/**
	 * Creates the buttons panel.
	 */
	protected JPanel createButtonPanel() {
		JPanel panel = new JPanel();
		panel.setLayout(new PaletteLayout(2, new Point(2,2), false));
		return panel;
	}

   void buildClassBar() {
      System.out.println("BUILDING CLASS BAR");

      if( classPalette == null ) {
         
         classPalette = new JPanel();
         classPalette.setLayout(new PaletteLayout(2,new Point(2,2)));

         Tool tool = createSelectionTool();
         classDefaultTool = createToolButton(IMAGES + "SEL", "Selection Tool", tool);
         classPalette.add(classDefaultTool);
         
         tool = new TextTool(this, new TextFigure());
         classPalette.add(createToolButton(IMAGES + "TEXT", "Text Tool", tool));
      
         tool = new NewClassFigureCreationTool(this);
         classPalette.add(createToolButton(JACIMAGES + "class", "Create a new class", tool));

         tool = new ClassFigureCreationTool(this);
         classPalette.add(createToolButton(JACIMAGES + "import", "Add an existing class", tool));
         tool = new RelationLinkShowTool(this);
         classPalette.add(createToolButton(JACIMAGES + "see", "Show existing relation", tool));
         
         tool = new RelationLinkCreationTool(this, new RelationLinkFigure());
         classPalette.add(createToolButton(JACIMAGES + "relation", "New relation link", tool));
         
         tool = new AggregationCreationTool(this, new RelationLinkFigure());
         classPalette.add(createToolButton(JACIMAGES + "aggregation", "Aggregation link", tool));         
         tool = new ConnectionTool(this, new RelationLinkFigure());
         classPalette.add(createToolButton(JACIMAGES + "addPoint", "Add a point", tool));
         
         //tool = new UndoableTool(new AttachedTextTool(this, AttachedTextFigure.createEndCardinality()));
         //classPalette.add(createToolButton(JACIMAGES + "relationCard", "Relation cardinality", tool));
         
         tool = new ConnectionTool(this, new InheritanceLinkFigure());
         classPalette.add(createToolButton(JACIMAGES + "inheritancelink", "Inheritance link", tool));      
         tool = new FieldCreationTool(this);
         classPalette.add(createToolButton(JACIMAGES + "attribute", "Create new attribute", tool));
         
         tool = new MethodCreationTool(this);
         classPalette.add(createToolButton(JACIMAGES + "method", "Create new method", tool));
      }
      if( currentPalette != null )
         getContentPane().remove(currentPalette);
      getContentPane().add("West", classPalette);
      currentPalette = classPalette;
      fDefaultToolButton = classDefaultTool;
      paint(getGraphics());
      validate();
      //      classPalette.paint(classPalette.getGraphics());
   }

   void buildAspectBar() {
      System.out.println("BUILDING ASPECT BAR");
      if( aspectPalette == null ) {
         
         aspectPalette = new JPanel();
         aspectPalette.setLayout(new PaletteLayout(2,new Point(2,2)));

         Tool tool = createSelectionTool();
         aspectDefaultTool = createToolButton(IMAGES + "SEL", "Selection Tool", tool);
         aspectPalette.add(aspectDefaultTool);
         
         tool = new TextTool(this, new TextFigure());
         aspectPalette.add(createToolButton(IMAGES + "TEXT", "Text Tool", tool));
      
         tool = new NewAspectFigureCreationTool(this);
         aspectPalette.add(createToolButton(JACIMAGES + "aspect", "Create a new aspect", tool));

         tool = new AspectFigureCreationTool(this);
         aspectPalette.add(createToolButton(JACIMAGES + "importAspect", "Add an existing aspect", tool));

         tool = new PointcutLinkCreationTool(this, new PointcutLinkFigure());
         aspectPalette.add(createToolButton(JACIMAGES + "pointcut", "New pointcut link", tool));
         
         tool = new PointcutLinkShowTool(this);
         aspectPalette.add(createToolButton(JACIMAGES + "seePointcut", "Show existing pointcut", tool));
         
         tool = new ConnectionTool(this, new RelationLinkFigure());
         aspectPalette.add(createToolButton(JACIMAGES + "addPoint", "Add a point", tool));
         
         tool = new FieldCreationTool(this);
         aspectPalette.add(createToolButton(JACIMAGES + "attribute", "Create new attribute", tool));
         
         tool = new MethodCreationTool(this);
         aspectPalette.add(createToolButton(JACIMAGES + "method", "Create new configuration method", tool));

         tool = new MethodCreationTool(this);
         aspectPalette.add(createToolButton(JACIMAGES + "aspectMethod", "Create new aspect method", tool));

      }
      if( currentPalette != null )
         getContentPane().remove(currentPalette);
      getContentPane().add("West", aspectPalette);
      currentPalette = aspectPalette;
      fDefaultToolButton = aspectDefaultTool;
      //resize(getSize());
      paint(getGraphics());
      validate();
   }

   void buildInstanceBar() {
      System.out.println("BUILDING INSTANCE BAR");
      if( instancePalette == null ) {
         
         instancePalette = new JPanel();
         instancePalette.setLayout(new PaletteLayout(2,new Point(2,2)));

         Tool tool = createSelectionTool();
         instanceDefaultTool = createToolButton(IMAGES + "SEL", "Selection Tool", tool);
         instancePalette.add(instanceDefaultTool);
         
         tool = new TextTool(this, new TextFigure());
         instancePalette.add(createToolButton(IMAGES + "TEXT", "Text Tool", tool));
      
         tool = new NewInstanceFigureCreationTool(this);
         instancePalette.add(createToolButton(JACIMAGES + "instance", "Create a new instance", tool));

         tool = new InstanceFigureCreationTool(this);
         instancePalette.add(createToolButton(JACIMAGES + "importInstance", "Add an existing instance", tool));
         
      }
      if( currentPalette != null )
         getContentPane().remove(currentPalette);
      getContentPane().add("West", instancePalette);
      currentPalette = instancePalette;
      fDefaultToolButton = instanceDefaultTool;
      //resize(getSize());
      paint(getGraphics());
      validate();

   }

   void buildGroupBar() {
      System.out.println("BUILDING GROUP BAR");
      if( groupPalette == null ) {
         
         groupPalette = new JPanel();
         groupPalette.setLayout(new PaletteLayout(2,new Point(2,2)));

         Tool tool = createSelectionTool();
         groupDefaultTool = createToolButton(IMAGES + "SEL", "Selection Tool", tool);
         groupPalette.add(groupDefaultTool);
         
         tool = new TextTool(this, new TextFigure());
         groupPalette.add(createToolButton(IMAGES + "TEXT", "Text Tool", tool));
      
         tool = new NewInstanceFigureCreationTool(this);
         groupPalette.add(createToolButton(JACIMAGES + "group", "Create a new group", tool));

         tool = new InstanceFigureCreationTool(this);
         groupPalette.add(createToolButton(JACIMAGES + "importGroup", "Add an existing group", tool));
         
      }
      if( currentPalette != null )
         getContentPane().remove(currentPalette);
      getContentPane().add("West", groupPalette);
      currentPalette = groupPalette;
      fDefaultToolButton = groupDefaultTool;
      //resize(getSize());
      paint(getGraphics());
      validate();
   }

	/**
	 * Creates the buttons shown in the buttons panel. Override to
	 * add additional buttons.
	 * @param panel the buttons panel.
	 */
	protected void createButtons(JPanel panel) {
		panel.add(new Filler(24,20));
                /*
		JComboBox drawingChoice = new JComboBox();
		drawingChoice.addItem(fgUntitled);

		String param = getParameter("DRAWINGS");
		if (param == null) {
			param = "";
		}
		StringTokenizer st = new StringTokenizer(param);
		while (st.hasMoreTokens()) {
			drawingChoice.addItem(st.nextToken());
		}
		// offer choice only if more than one
		if (drawingChoice.getItemCount() > 1) {
			panel.add(drawingChoice);
		}
		else {
			panel.add(new JLabel(fgUntitled));
		}

		drawingChoice.addItemListener(
			new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					if (e.getStateChange() == ItemEvent.SELECTED) {
						loadDrawing((String)e.getItem());
					}
				}
			}
		);
                */
                JLabel title = new JLabel(substance.toString());
                panel.add(title);

		panel.add(new Filler(6,20));

                JComboBox combo = new JComboBox(new Object[] {
                   "class mode",
                   "aspect mode",
                   "instance mode",
                   "group mode"}
                );

		combo.addActionListener(
                   new ActionListener() {
                         public void actionPerformed(ActionEvent event) {
                            JComboBox cb = (JComboBox)event.getSource();
                            String command = (String)cb.getSelectedItem();
                            if( command.equals("class mode") ) {
                               buildClassBar();
                            } else if( command.equals("aspect mode") ) {
                               buildAspectBar();
                            } else if( command.equals("instance mode") ) {
                               buildInstanceBar();
                            } else if( command.equals("group mode") ) {
                               buildGroupBar();
                            }
                            setSelected(fDefaultToolButton);
                         }
                      }
		);

                panel.add(combo);
                
		JButton button;
		//button = new CommandButton(new SaveCommand("Save", this));
		//panel.add(button);

		/*button = new CommandButton(new DeleteCommand("Delete", this));
		panel.add(button);

		button = new CommandButton(new DuplicateCommand("Duplicate", this));
		panel.add(button);

		button = new CommandButton(new GroupCommand("Group", this));
		panel.add(button);

		button = new CommandButton(new UngroupCommand("Ungroup", this));
		panel.add(button);

		button = new JButton(new ImageIcon(Jac.getJacRoot()+"src/jac/ide/images/diagram/group.gif"));
		button.addActionListener(
			new ActionListener() {
				public void actionPerformed(ActionEvent event) {
					showHelp();
				}
			}
		);
		panel.add(button);
		button = new JButton(new ImageIcon(Jac.getJacRoot()+"src/jac/ide/images/diagram/ungroup.gif"));
		panel.add(button);
                */

		/*fUpdateButton = new JButton("Simple Update");
		fUpdateButton.addActionListener(
			new ActionListener() {
				public void actionPerformed(ActionEvent event) {
					if (fSimpleUpdate) {
						setBufferedDisplayUpdate();
					}
					else {
						setSimpleDisplayUpdate();
					}
				}
			}
                        );*/

		// panel.add(fUpdateButton); // not shown currently
	}

	/**
	 * Creates the tools palette.
	 */
	protected JPanel createToolPalette() {
		JPanel palette = new JPanel();
		palette.setLayout(new PaletteLayout(2,new Point(2,2)));
		return palette;
	}

	/**
	 * Creates the tools. By default only the selection tool is added.
	 * Override this method to add additional tools.
	 * Call the inherited method to include the selection tool.
	 * @param palette the palette where the tools are added.
	 */
	protected void createTools(JPanel palette) {
		Tool tool = createSelectionTool();

		fDefaultToolButton = createToolButton(IMAGES + "SEL", "Selection Tool", tool);
		palette.add(fDefaultToolButton);
	}

	/**
	 * Creates the selection tool used in this editor. Override to use
	 * a custom selection tool.
	 */
	protected Tool createSelectionTool() {
		return new SelectionTool(this);
	}

	/**
	 * Creates a tool button with the given image, tool, and text
	 */
	protected ToolButton createToolButton(String iconName, String toolName, Tool tool) {
		return new ToolButton(this, iconName, toolName, tool);
	}

	/**
	 * Creates the drawing used in this application.
	 * You need to override this method to use a Drawing
	 * subclass in your application. By default a standard
	 * Drawing is returned.
	 */
	protected Drawing createDrawing() {
		return new StandardDrawing();
	}

	/**
	 * Creates the drawing view used in this application.
	 * You need to override this method to use a DrawingView
	 * subclass in your application. By default a standard
	 * DrawingView is returned.
	 */
	protected DrawingView createDrawingView() {
		return new StandardDrawingView(this, 2000, 2000);
	}

	/**
	 * Handles a user selection in the palette.
	 * @see PaletteListener
	 */
	public void paletteUserSelected(PaletteButton button) {
		ToolButton toolButton = (ToolButton) button;
		setTool(toolButton.tool(), toolButton.name());
		setSelected(toolButton);
	}

	/**
	 * Handles when the mouse enters or leaves a palette button.
	 * @see PaletteListener
	 */
	public void paletteUserOver(PaletteButton button, boolean inside) {
		if (inside) {
			showStatus(((ToolButton) button).name());
		}
		else {
			showStatus(fSelectedToolButton.name());
		}
	}

	/**
	 * Gets the current drawing.
	 * @see DrawingEditor
	 */
	public Drawing drawing() {
		return fDrawing;
	}

	/**
	 * Gets the current tool.
	 * @see DrawingEditor
	 */
	public Tool tool() {
		return fTool;
	}

	/**
	 * Gets the current drawing view.
	 * @see DrawingEditor
	 */
	public DrawingView view() {
		return fView;
	}

	public DrawingView[] views() {
		return new DrawingView[] { view() } ;
	}

	/**
	 * Sets the default tool of the editor.
	 * @see DrawingEditor
	 */
	public void toolDone() {
		setTool(fDefaultToolButton.tool(), fDefaultToolButton.name());
		setSelected(fDefaultToolButton);
	}

	/**
	 * Handles a change of the current selection. Updates all
	 * menu items that are selection sensitive.
	 * @see DrawingEditor
	 */
	public void figureSelectionChanged(DrawingView view) {
           //setupAttributes();
	}

	public void viewSelectionChanged(DrawingView oldView, DrawingView newView) {
	}

	private void initDrawing() {
		fDrawing = createDrawing();
		view().setDrawing(fDrawing);
		toolDone();
	}

	private void setTool(Tool t, String name) {
		if (fTool != null) {
			fTool.deactivate();
		}
		fTool = t;
		if (fTool != null) {
			showStatus(name);
			fTool.activate();
		}
	}

	private void setSelected(ToolButton button) {
		if (fSelectedToolButton != null) {
			fSelectedToolButton.reset();
		}
		fSelectedToolButton = button;
		if (fSelectedToolButton != null) {
			fSelectedToolButton.select();
		}
	}

	protected void loadDrawing(String param) {
		if (param == fgUntitled) {
			fDrawing.release();
			initDrawing();
			return;
		}

		String filename = getParameter(param);
		if (filename != null) {
			readDrawing(filename);
		}
	}

	private void readDrawing(String filename) {
		toolDone();
		String type = guessType(filename);
		if (type.equals("storable")) {
			readFromStorableInput(filename);
		}
		else if (type.equals("serialized")) {
			readFromObjectInput(filename);
		}
		else {
			showStatus("Unknown file type");
		}
	}

	private void readFromStorableInput(String filename) {
		try {
			URL url = new URL(getCodeBase(), filename);
			InputStream stream = url.openStream();
			StorableInput input = new StorableInput(stream);
			fDrawing.release();

			fDrawing = (Drawing)input.readStorable();
			view().setDrawing(fDrawing);
		}
		catch (IOException e) {
			initDrawing();
			showStatus("Error:" + e);
		}
	}

	private void readFromObjectInput(String filename) {
		try {
			URL url = new URL(getCodeBase(), filename);
			InputStream stream = url.openStream();
			ObjectInput input = new ObjectInputStream(stream);
			fDrawing.release();
			fDrawing = (Drawing)input.readObject();
			view().setDrawing(fDrawing);
		}
		catch (IOException e) {
			initDrawing();
			showStatus("Error: " + e);
		}
		catch (ClassNotFoundException e) {
			initDrawing();
			showStatus("Class not found: " + e);
		}
	}

	private String guessType(String file) {
		if (file.endsWith(".draw")) {
			return "storable";
		}
		if (file.endsWith(".ser")) {
			return "serialized";
		}
		return "unknown";
	}

	private void setupAttributes() {
		Color   frameColor = (Color)   AttributeFigure.getDefaultAttribute("FrameColor");
		Color   fillColor  = (Color)   AttributeFigure.getDefaultAttribute("FillColor");
		Color   textColor  = (Color)   AttributeFigure.getDefaultAttribute("TextColor");
		Integer arrowMode  = (Integer) AttributeFigure.getDefaultAttribute("ArrowMode");
		String  fontName   = (String)  AttributeFigure.getDefaultAttribute("FontName");

		FigureEnumeration k = view().selectionElements();
		while (k.hasMoreElements()) {
			Figure f = k.nextFigure();
			frameColor = (Color) f.getAttribute("FrameColor");
			fillColor  = (Color) f.getAttribute("FillColor");
			textColor  = (Color) f.getAttribute("TextColor");
			arrowMode  = (Integer) f.getAttribute("ArrowMode");
			fontName   = (String) f.getAttribute("FontName");
		}

		fFrameColor.setSelectedIndex(ColorMap.colorIndex(frameColor));
		fFillColor.setSelectedIndex(ColorMap.colorIndex(fillColor));
		//fTextColor.select(ColorMap.colorIndex(textColor));
		if (arrowMode != null) {
			fArrowChoice.setSelectedIndex(arrowMode.intValue());
		}
		if (fontName != null) {
			fFontChoice.setSelectedItem(fontName);
		}
	}

	protected void setSimpleDisplayUpdate() {
		view().setDisplayUpdate(new SimpleUpdateStrategy());
		fUpdateButton.setText("Simple Update");
		fSimpleUpdate = true;
	}

	protected void setBufferedDisplayUpdate() {
		view().setDisplayUpdate(new BufferedUpdateStrategy());
		fUpdateButton.setText("Buffered Update");
		fSimpleUpdate = false;
	}

	/**
	 * Shows a help page for the applet. The URL of the help
	 * page is derived as follows: codeBase+appletClassname+Help.html"
	 */
	protected void showHelp() {
		try {
			String appletPath = getClass().getName().replace('.', '/');
			URL url = new URL(getCodeBase(), appletPath + "Help.html");
			getAppletContext().showDocument(url, "Help");
		}
		catch (IOException e) {
			showStatus("Help file not found");
		}

	}

	protected void setUndoManager(UndoManager newUndoManager) {
		myUndoManager = newUndoManager;
	}
	
	public UndoManager getUndoManager() {
		return myUndoManager;
	}

	protected VersionControlStrategy getVersionControlStrategy() {
		return new StandardVersionControlStrategy(this);
	}

	/**
	 * Subclasses should override this method to specify to which versions of
	 * JHotDraw they are compatible. A string array is returned so it is possible
	 * to specify several version numbers of JHotDraw to which the application
	 * is compatible with.
	 *
	 * @return all versions number of JHotDraw the application is compatible with.
	 */
	public String[] getRequiredVersions() {
		String[] requiredVersions = new String[1];
		// return the version of the package we are in
		requiredVersions[0] = VersionManagement.getPackageVersion(DiagramApplet.class.getPackage());
		return requiredVersions;
	}

	/**
	 * *** netscape browser work around ***
	 */
	private void startSleeper() {
		if (fSleeper == null) {
			fSleeper = new SleeperThread(this);
		}
		fSleeper.start();
	}

	private void stopSleeper() {
		if (fSleeper != null) {
			fSleeper.stop();
		}
	}


   public void save() {
      if( substance == null ) {
         Log.error("no subtance");
         return;
      }
      String filename = Jac.getJacRoot()+"ideData/"+
         ((jac.ide.Diagram)substance).getContainer().getPath()+"/"+
         ((jac.ide.Diagram)substance).getName()+".draw";
      try {
         StorageFormat foundFormat = new StandardStorageFormat();
         if (foundFormat != null) {
            saveDrawing(foundFormat, filename);
         }
         else {
            showStatus("Not a valid file format: " + "draw");
         }
      }
      catch (Exception e) {
         //initDrawing();
         e.printStackTrace();
         showStatus("Error:" + e);
      }
   }

   public void load() {
      toolDone();
      if( substance == null ) {
         Log.error("no subtance");
         return;
      }
      String filename = Jac.getJacRoot()+"ideData/"+
         ((jac.ide.Diagram)substance).getContainer().getPath()+"/"+
         ((jac.ide.Diagram)substance).getName()+".draw";
      
      try {

         System.out.println("reading drawing");
         DiagramView.init = true;
         URL url = new URL("file:"+filename);
         
         InputStream stream = url.openStream();
         StorableInput input = new StorableInput(stream);
         fDrawing.release();
         
         fDrawing = (Drawing)input.readStorable();
         view().setDrawing(fDrawing);
      }
      catch (IOException e) {
         //initDrawing();
         e.printStackTrace();
         showStatus("Error:" + e);
      } finally {
         DiagramView.init = false;  
      }
      
   }


   /**
    * Shows a file dialog and opens a drawing.
    */
   public void promptOpen() {
      toolDone();
      JFileChooser openDialog = createOpenFileChooser();
      getStorageFormatManager().registerFileFilters(openDialog);
      if (openDialog.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
         String filename = openDialog.getSelectedFile().getAbsolutePath();
         try {
            
            System.out.println("reading drawing");
            URL url = new URL("file:"+filename);

            InputStream stream = url.openStream();
            StorableInput input = new StorableInput(stream);
            fDrawing.release();
            
            fDrawing = (Drawing)input.readStorable();
            view().setDrawing(fDrawing);
            

         }
         catch (IOException e) {
            //initDrawing();
            e.printStackTrace();
            showStatus("Error:" + e);
         }
         /*
         String type = guessType(filename);
         if (type.equals("storable")) {
            readFromStorableInput(filename);
         }
         else if (type.equals("serialized")) {
            readFromObjectInput(filename);
         }
         else {
            showStatus("Unknown file type");
            }*/

         /*StorageFormat foundFormat = getStorageFormatManager().findStorageFormat(openDialog.getFileFilter());
         if (foundFormat != null) {
            loadDrawing(foundFormat, openDialog.getSelectedFile().getAbsolutePath());
         }
         else {
            showStatus("Not a valid file format: " + openDialog.getFileFilter().getDescription());
            }*/
      }
   }


   public void promptSave() {
      toolDone();
      JFileChooser saveDialog = createOpenFileChooser();
      getStorageFormatManager().registerFileFilters(saveDialog);
      if (saveDialog.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
         String filename = saveDialog.getSelectedFile().getAbsolutePath();
         try {
      
            StorageFormat foundFormat = getStorageFormatManager().findStorageFormat(saveDialog.getFileFilter());
            if (foundFormat != null) {
               saveDrawing(foundFormat, saveDialog.getSelectedFile().getAbsolutePath());
            }
            else {
               showStatus("Not a valid file format: " + saveDialog.getFileFilter().getDescription());
            }
         }
         catch (Exception e) {
            //initDrawing();
            e.printStackTrace();
            showStatus("Error:" + e);
         }
         /*
         String type = guessType(filename);
         if (type.equals("storable")) {
            readFromStorableInput(filename);
         }
         else if (type.equals("serialized")) {
            readFromObjectInput(filename);
         }
         else {
            showStatus("Unknown file type");
            }*/

         /*StorageFormat foundFormat = getStorageFormatManager().findStorageFormat(openDialog.getFileFilter());
         if (foundFormat != null) {
            loadDrawing(foundFormat, openDialog.getSelectedFile().getAbsolutePath());
         }
         else {
            showStatus("Not a valid file format: " + openDialog.getFileFilter().getDescription());
            }*/
      }
   }

	protected void saveDrawing(StorageFormat storeFormat, String file) {
		// Need a better alert than this.
		if(view() == null) {
			return;
		}
		try {
                   File f = new File(file);
                   if ( f.exists() )
                      f.delete();
                   f.getParentFile().mkdirs();
			String name = storeFormat.store(file, view().drawing());
			view().drawing().setTitle(name);
			//setDrawingTitle(name);
		}
		catch (IOException e) {
			showStatus(e.toString());
		}
	}


   protected JFileChooser createOpenFileChooser() {
      JFileChooser openDialog = new JFileChooser();
      openDialog.setDialogTitle("Open File...");
      return openDialog;
   }

   public void showStatus(String msg) {
      ((CustomizedGUI)DiagramView.display).showStatus(msg);
   }

}


class SleeperThread extends Thread {

	JApplet  fApplet;

	SleeperThread(JApplet applet) {
		fApplet = applet;
	}

	public void run() {
		try {
			for (;;) {
				fApplet.showStatus("loading icons...");
				sleep(50);
			}
		}
		catch (InterruptedException e) {
			return;
		}
	}

}

