
package jac.ide.diagrams;

import java.awt.*;
import java.util.*;
import java.io.IOException;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.figures.*;
import CH.ifa.draw.standard.*;
import CH.ifa.draw.util.UndoableAdapter;
import CH.ifa.draw.util.Undoable;
import CH.ifa.draw.util.StorableOutput;
import CH.ifa.draw.util.StorableInput;
import java.awt.event.MouseEvent;

import jac.ide.RelationLink;
import jac.core.NameRepository;
import jac.util.Log;
import jac.aspects.gui.View;

public class LinkFigure extends LineConnection 
   implements View {

   Vector attachedFigures = new Vector();

   public void addAttachedTextFigure(AttachedTextFigure f) {
      attachedFigures.add(f);
   }

   public AttachedTextFigure createAttachedFigure(int type) {
      AttachedTextFigure at = new AttachedTextFigure(type);
      at.setSubstance(substance);
      at.connect(this);
      at.refresh();
      return at;    
   }

   public AttachedTextFigure createAttachedFigure(int type,String s) {
      AttachedTextFigure at = new AttachedTextFigure(type);
      at.setSubstance(substance);
      at.connect(this);
      at.setText(s);
      return at;    
   }

   public AttachedTextFigure createName(String s) {
      return createAttachedFigure(AttachedTextFigure.NAME,s);
   }

   public AttachedTextFigure createEndRole(String s) {
      return createAttachedFigure(AttachedTextFigure.END_ROLE,s);
   }

   public AttachedTextFigure createStartRole(String s) {
      return createAttachedFigure(AttachedTextFigure.START_ROLE,s);
   }

   public AttachedTextFigure createStartCardinality(String s) {
      return createAttachedFigure(AttachedTextFigure.START_CARDINALITY,s);
   }

   public AttachedTextFigure createEndCardinality(String s) {
      return createAttachedFigure(AttachedTextFigure.END_CARDINALITY,s);
   }

   jac.ide.RoledLink substance;

   // View interface
   public View getParentView() {
      return null;
   }

   public void refreshViewItem(jac.core.rtti.FieldItem field,
                               jac.core.rtti.MethodItem method, Object[] args) {
      refreshView();
   }

   public void setFocus(jac.core.rtti.FieldItem field, Object extraOption) {}

   public void refreshView() {
      if( DiagramView.init ) return;
      DiagramView.init = true;

      Iterator it = attachedFigures.iterator();
      while( it.hasNext() ) {
         AttachedTextFigure cur = (AttachedTextFigure)it.next();
         cur.refresh();
      }
      if(substance.getEnd() != ((View)endFigure()).getSubstance()) {
         Figure newEnd = ((DiagramView)DiagramView.diagramViews.get(0))
            .findElement((jac.ide.ModelElement)substance.getEnd());
         if( newEnd != null ) {
            Log.trace("figures","reconnecting relation to "+newEnd);
            disconnectEnd();
            endPoint(newEnd.center().x,newEnd.center().y);
            connectEnd(newEnd.connectorAt(
               newEnd.center().x,newEnd.center().y));
            updateConnection();
         } else {
            ((DiagramView)DiagramView.diagramViews.get(0)).view().drawing().remove(this);            
         }
      }
      if(substance instanceof RelationLink) {
         if(((RelationLink)substance).isAggregation()) {
            setStartDecoration(new ArrowTip(0.50, 15, 28));
            setEndDecoration(new ArrowTip());
         } else {
            setStartDecoration(null);
            setEndDecoration(null);
         }
      }

      ((DiagramView)DiagramView.diagramViews.get(0)).view().paint(((DiagramView)DiagramView.diagramViews.get(0)).view().getGraphics());
      DiagramView.init = false;
   }

   public void close() {}
   
   /**
    * Get the value of substance.
    * @return value of substance.
    */
   public Object getSubstance() {
      return substance;
   }
   
   /**
    * Set the value of substance.
    * @param v  Value to assign to substance.
    */
   public void setSubstance(jac.ide.RoledLink  v) {
      this.substance = v;
   }
   
   public LinkFigure() {
      setStartDecoration(null);
      setEndDecoration(null);
   }

   public void handleConnect(Figure start, Figure end) {
      if( substance == null ) return;
      if( ((View)end).getSubstance() != substance.getEnd() ) {
         DiagramView.init = true;
         substance.setEnd((jac.ide.ModelElement)((View)end).getSubstance());
         DiagramView.init = false;
      }
   }

   public void handleDisconnect(Figure start, Figure end) {
   } 

   public Vector handles() {
      Vector handles = super.handles();
      // don't allow to reconnect the starting figure
      handles.setElementAt(new NullHandle(this, PolyLineFigure.locator(0)), 0);
      return handles;
   }


   public void write(StorableOutput dw) {
      super.write(dw);

      dw.writeString(NameRepository.get().getName(substance));

      /*
      Iterator it = attachedFigures.iterator();
      while( it.hasNext() ) {
         AttachedTextFigure cur = (AttachedTextFigure)it.next();
         dw.writeInt(cur.getType());
      }*/
   }

   public void read(StorableInput dr) throws IOException {
      super.read(dr);

      try {
         String name = dr.readString();
         substance = (jac.ide.RoledLink)NameRepository.get().getObject(name);
      } catch(Exception e) {
         e.printStackTrace();
      }

      ((jac.core.Wrappee)substance).invokeRoleMethod("addView",new Object[]{this});
      
      /*try {
         while(true) {
            attachedFigures.add(createAttachedFigure(dr.readInt()));            
         }
      } catch(Exception e) {}
      */
   }

   public Locator connectedTextLocator(Figure f) {
      return new AttachedTextLocator();
   }

   public void selectAll(DrawingView v) {
      Iterator it = attachedFigures.iterator();
      while( it.hasNext() ) {
         AttachedTextFigure cur = (AttachedTextFigure)it.next();
         v.addToSelection(cur);
      }
   }

}
