/*

  Copyright (C) 2001 Renaud Pawlak

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

package jac.wrappers;

import jac.core.*;
import java.lang.reflect.*;
import java.util.*;

/**
 * This wrapper prints some informations about the wrapped method when
 * it is called. */

public class VerboseWrapper extends Wrapper {

   /**
    * A wrapping method that prints out the calling informations.
    *
    * @return the value of the called method */

   public Object printCallingInfos() {

      /** Before code ... */
      Date d1 = new Date();
      System.out.println(
         "<<< " + (d1.getTime() - JacObject.getStartTime().getTime()) + 
         " ms: calling '" + method() + "' on '" + 
         NameRepository.get().getName(wrappee()) +
         "', args = " + Arrays.asList( args() ) + " >>>"
      );
      /** End before code */

      Object ret = proceed();

      /** After code ... */
      Date d2 = new Date();
      System.out.println(
         "<<< " + (d2.getTime() - JacObject.getStartTime().getTime()) + 
         " ms: returning '" + method() + "' on '" + 
         NameRepository.get().getName(wrappee()) +
         "', value = " + ret + ", duration = " +
         (d2.getTime() - d1.getTime()) + " ms >>>"
      );
      /** End after code */

      return ret;
   }
}


